package grada;

import grada.event.*;
import grada.geometriefiguren.*;
import grada.interfaces.*;
import grada.steuereinheit.*;
import parser.*;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

import sprachen.Sprachen;


/** Bedienoberflche fr die Funktionenanzeige und -Eingabe.
 * 
 * In dieser Klasse ist die Benutzeroberflche untergebracht, hier reagiert die
 * Oberflche dynamisch auf die Eingaben des Benutzers, mit vorgegebenen 
 * Funktionstermen kann man sehr einfache Funktionen anzeigen, aber auch eigene 
 * komplexere Funktionen eingeben. 
 * @author Thilo Beckmann
 * @version 07.07.05
**/
public class FunktionenAnzeigen extends JPanel 
			implements 
			InterfaceParserEingabe,				
			//Damit Parser Fehler melden kann
			InterfaceFunktionsListenAenderung,	
			//Damit nderungen an der FunktionsListe bernommen werden
			InterfaceZoomAenderung,				
			//Damit Zoomnderungen bernommen werden knnen
			InterfaceFunktionsEingabe,			
			//Damit die Eingabe von Funktionen gewhrleistet ist
			InterfaceFunktionsParameterAenderung,
			//Damit die nderungen der Parameter bernommen werden
			FocusListener
			{
	// fuer die generelle Aufteilung und Gestaltung:
	private JSplitPane trenner;
	private JScrollPane scrollfenster;
	private Schaubild s = new Schaubild(this);
	private JPanel p 	= new JPanel();
	
	// EventHaendler, der alle Events haendelt:
	private EventHaendler myEventHaendler = new EventHaendler();
	// Die Funktionsliste, in der sich alle Funktionen befinden. 
	private ArrayList	fuListe	= new ArrayList();
	// zur Steuerrung der Sprache
	private Sprachen	sprachen = Sprachen.holeSprache();
	private ListResourceBundle	sp = sprachen.holeSprachBundle();
	
	private static final Insets abstand		= new Insets (1,1,1,1);
	private static final Insets abstand0	= new Insets (0,0,0,0);
	
	
	/* ---------------------------------------- */
	/* --- Konstruktor (ueberladen): ---------- */
	/* ---------------------------------------- */
	/**
	 * FunktionenAnzeigen erstellt Benutzeroberflche.
	 * Dieser Konstruktor bentigt alle Informationen zur Oberflche. Es kann 
	 * alles wichtige eingestellt werden.
	 * @param funktionsterm		Funktionsterm, der in der Eingabezeile steht
	 */
	public FunktionenAnzeigen(String funktionsterm){
		setLayout(new GridLayout());
		scrollfenster	= new JScrollPane(p);
		trenner			= new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true,
				scrollfenster,s);
		add(trenner);
		p.setLayout(new GridBagLayout());
		s.addComponentListener(myEventHaendler);
		erstelleOberflaeche(funktionsterm);
		validate();
	}
	
	
	private FunktionsEingabeFeld eingabe		= new FunktionsEingabeFeld("",this);
	private FormVariablenScrollbalken scrbalken	= new FormVariablenScrollbalken(fuListe,this);
	private FunktionenAuswahl auswahl			= new FunktionenAuswahl(fuListe,this);
	private ZoomBedienung zoom					= new ZoomBedienung(s.holeVergroesserungsStufeX(),s.holeVergroesserungsStufeY(),this);
	
	/** Erstellt Benutzeroberflche. 
	 * hier werden die verschiedenen Bereiche der Benutzeroberflche erstellt.
	 * @param fterm	Funktionsterm, der am Anfang im FunktionsEingabeFeld steht
	 */
	private void erstelleOberflaeche(String fterm){
		eingabe.setzeEingabeText(fterm);
		GridBagHinzufueger.add(p,eingabe,		0,0,1,1,1,  0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
		GridBagHinzufueger.add(p,scrbalken,		0,1,1,1,1,  0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
		GridBagHinzufueger.add(p,auswahl,		0,2,1,1,1,  0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
		GridBagHinzufueger.add(p,new JLabel(""),0,3,1,1,1,999,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
		GridBagHinzufueger.add(p,zoom,			0,4,1,1,1,  0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
		revalidate();
	}
	/** Aktualisiert die Oberflche.
	 * Wenn eine Funktion hinzugefgt, oder verndert wurde, muss die 
	 * Oberflche aktualisiert werden, damit die Funktionen alle in der Liste 
	 * vorhanden sind bzw. die Scrollbalken alle sichtbar sind.
	 */
	private void aktualisiereOberflaeche(){
		// eingabe bleibt ohne nderung
		scrbalken	.setzeFunktionsListe(fuListe);
		scrbalken	.revalidate();
		auswahl		.setzeFunktionsListe(fuListe);
		auswahl		.revalidate();
		// zoom bleibt ohne nderung
		s.zeichne();
		revalidate();
	}
	
	/** FunktionenAnzeigen erstellt Benutzeroberflche.
	 * Dieser Konstruktor bentigt alle Informationen zur Oberflche. Es kann 
	 * alles wichtige  eingestellt werden. Scrollbalken, Zoom, Optionen und 
	 * FunktionsEingabe werden angezeigt, es wird keine Funktion ins 
	 * Eingabefeld vorgegeben.
	 */
	public FunktionenAnzeigen(){
		this("");
	}
	
	/** Ruft DateiMen auf.
	 * Hier wird ein Dateiname und ein Pfad ausgewhlt. Die Datei wird dann
	 * dorthin gespeichert.
	 */
	public void speicherFunkionenInDatei(){
		JFileChooser fileChooser = new JFileChooser();
		fileChooser.setFileFilter(new DateiFilter());
		if (fileChooser.showSaveDialog(null) == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			String dateiname = file.getAbsolutePath();
			if (!dateiname.endsWith(sp.getString("DateiFilter_DateiEndung")))
				dateiname += sp.getString("DateiFilter_DateiEndung"); 
			speicherFunkionenInDatei(dateiname);
		}
	}
	
	/** Speichert Daten in Datei.
	 * Diese methode sorgt dafr, dass die Funktionen und Beschriftungen, die
	 * gerade eingestellt sind, in die anzugebene Datei gespeichert werden. 
	 * 
	 * @param dateiname		Dateiname/(pfad)
	 */
	public void speicherFunkionenInDatei(String dateiname){
		FileOutputStream fileOutputStream;
		ObjectOutputStream schreiber;
		try{
			fileOutputStream = new FileOutputStream(dateiname);
			schreiber = new ObjectOutputStream(fileOutputStream);
			schreiber.writeObject(fuListe.clone());
			schreiber.close();
		}
		catch (FileNotFoundException e){}
		catch (IOException e){}	
	}
	
	/** Zeigt Auswahlbildschirm.
	 * Hier wird ein Auswahlbildschirm gezeigt, in dem man eine Datei auswhlen 
	 * kann, die zuvor gespeichert wurde.
	 */
	public void ladeFunktionenAusDatei(){
		JFileChooser fileChooser = new JFileChooser();
		
		fileChooser.setFileFilter(new DateiFilter());
		if (fileChooser.showOpenDialog(null) == JFileChooser.APPROVE_OPTION) {
			File file = fileChooser.getSelectedFile();
			ladeFunktionenAusDatei(file.getAbsolutePath());
		}
	}
	
	/** Ld eine Datei.
	 * Alle Funktionen und Beschriftungen werden aus der anzugebenden Datei 
	 * geholt.
	 * @param dateiname	Dateiname/(pfad)
	 */
	public void ladeFunktionenAusDatei(String dateiname){
		try{
			ObjectInputStream leser;
			FileInputStream fIStream = new FileInputStream(dateiname);
			BufferedInputStream bInStream = new BufferedInputStream(fIStream);
			leser = new ObjectInputStream(bInStream);
			fuListe = (ArrayList)leser.readObject();
			erstelleOberflaeche("");
			aktualisiere();
		}
		catch(FileNotFoundException e)  {}
		catch(IOException e)			{}
		catch(ClassNotFoundException e) {}
	}
	
	/** Setzt die Eingabezeile.
	 * Hiermit kann der Parser die zu parsenden Funktionstext in eine korrekte 
	 * Form gebracht in das Eingabefeld schreiben.
	 * @param term	korrigierter Term vom Parser
	 */
	public void setzeTermEingabe(String term){
		//eingabe.setzeEingabeText(term);
	}
	/** Gibt die Parameter zurck.
	 * Hiermit kann der Parser sich die genderten Werte der Funktion holen, 
	 * um somit schneller die neuen Punkte fr die Graphenanzeige im Schaubild 
	 * zu berechnen.
	 * @return	gibt ein Array der Lnge 26 zurck, in dem alle in Scrollbalken 
	 * 			eingestellten Formvariablen gespeichert sind. 
	 */
	public double[] holeTermParameter(){
		final double	d[]	= new double[scrbalken.MAXSCR];
		JScrollBar scr[] = scrbalken.holeScrollbalken();
		for (int i=0; i<scrbalken.MAXSCR; i++){
			d[i] = 0.0;
			try {
				d[i] = (double)scr[i].getValue()/100.0;
			}
			catch (Exception e){}
		}
		return d;
	}
	
	/** Gibt Zusatzinformationen.
	 * Hiermit kann der Parser eine Information fr den Benutzer bergeben 
	 * (ntzlich z.B. wenn eine falsche Eingabe korrigiert wurde und mit 
	 * dieser Funktion dieser Hinweis als Tooltip angezeigt wird.
	 * @param info	Text mit Informationen
	 */
	public void setzeTermInformation(String info){
	}
	
	/** Gibt Fehlerinformation.
	 * hier wird mitgeteilt, ob eine eingegebene Funktion einen Fehler 
	 * aufweist, und der vom Benutzer korrigierte Funktionsterm mitgeteilt.
	 * @param fehlerhafteFunktion	alte fehlerhafte Funktion
	 * @param korrigierterTerm		neue korrigierte Funktion
	 */
	public void funktionFehlerhaft(	String fehlerhafteFunktion,  // TODO
									String korrigierterTerm){
		System.out.println("FEHLER BEI DER FUNKTIONS EINGABE: "+
				fehlerhafteFunktion+ " ___ neu: "+korrigierterTerm);
		for (int i=0; i< fuListe.size(); i++){
			Iterator iter = fuListe.iterator();
			while(iter.hasNext()){
				Funktion fu = (Funktion)iter.next(); 
				if (fu.holeFunktionsTerm() == fehlerhafteFunktion){
					fu.setzeFunktionsTerm(korrigierterTerm);
					eingabe.setzeEingabeText(korrigierterTerm);
					aktualisiereOberflaeche();
				}
			}
		}
	}
	
	/** Aktualisiert die Anzeige.
	 * Diese Methode wird vom Programm (Frame) aufgerufen, falls dieses in 
	 * der Gre oder Position gendert wurde, damit das Schaubild des Graphen 
	 * nicht teilweise oder komplett gelscht ist. 
	 */
	public void aktualisiere(){
		s.gebeFunktionen(fuListe);
		if (Parser.aktualisierenErlaubt){
			s.aktualisiere();
		}
		s.zeichne();
	}
	
	/** Diese Methode wird aufgerufen, wenn mit Hilfe der FunktionsAuswahl die 
	 * Liste der Funktionen verndert wurde, sei es durch lschen oder 
	 * verndern einer Funktion.
	 * @param e	FunktionsListenEvent, das die neue FunktionsListe enthlt.
	 */
	public void funktionsListeWurdeVeraendert(FunktionsListenEvent e){
		this.fuListe = e.holeFunktionsListe();
		aktualisiereOberflaeche();
	}
	
	/** Diese Methode wird aufgerufen, wenn an einem der Scrollbalken ein Wert 
	 * fr eine Formvariable gendert wurde
	 */
	public void funktionsParameterWurdenVeraendert(FunktionsParameterEvent e){
		s.zeichne();
	}
	
	/** Diese Methode wird aufgerufen, wenn einer der Button in der 
	 * ZoomBedienung angeklickt und damit der Zoom verndert wurde 
	 * (aufgefordert wird den Zoom zu ndern).
	 * @param e		ZoomEvent, das die neuen Zoomstufen enthlt
	 */
	public void zoomWurdeVeraendert(ZoomEvent e){
		s.setzeVergroesserungsStufeX (e.holeXzoom());
		s.setzeVergroesserungsStufeY (e.holeYzoom());
		s.zeichne();
	}
	
	/** Dieses Methode wird aufgerufen, wenn in im FunktionsEingabeFeld eine
	 * Funktion eingegeben und dann mit Enter oder mit dem Button aufgefordert 
	 * wird, diesen Term als Graph zu zeichnen.
	 * @param e	FunktionsEingabeEvent, in dem der eingegebene Term steckt.
	 */
	public void funktionsTermIstEingegebenWorden(FunktionsEingabeEvent e){
		fuListe.add(new Funktion(e.holeTerm(),false, true, Color.red, 
				new ArrayList()));
		aktualisiereOberflaeche();
	}
	
	public void focusGained(FocusEvent e){
		
	}
	public void focusLost (FocusEvent e){
		
	}
	
	
	/** Diese Klasse regelt alle vorkommenden Events.
	 * Alle Events die von den existierenden Objekten zu behandeln sind, 
	 * werden hier verarbeitet.
	 * @author Thilo Beckmann
	 * @version 05.07.05
	 */
	class EventHaendler implements ComponentListener{
		public void componentHidden(ComponentEvent e){}
		public void componentMoved(ComponentEvent e){}
		public void componentResized(ComponentEvent e){aktualisiere();}
		public void componentShown(ComponentEvent e){}
	}
}