package grada;

import grada.event.ZoomEvent;
import grada.geometriefiguren.Funktion;
import grada.interfaces.InterfaceJFrameAenderung;
import grada.interfaces.InterfaceParserEingabe;
import grada.interfaces.InterfaceZoomAenderung;
import grada.steuereinheit.GridBagHinzufueger;
import grada.steuereinheit.ZoomBedienung;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import parser.*;
import sprachen.Sprachen;


/**
 * @author Felix
 * @version 1.0 (vom 11.09.2005)
 */
/**Mit Hilfe von dieer Klasse kann man Polynome bis max 10ten grades durcheinander teilen.
 * Unter anderem kan man sich Aufgaben generieren lassen, um selber die Polynomdivision zu lernen.
 * Generell dient dises Programm dem erlernen des Umgangs mit der Polynomdivision.
 * Zu dieem Zweck ist es auch Hilfreich an den Graphen zu sehen, was der unterschied zwischen
 * Zaehler und Ergebnis ist.
 */
public class PolynomDivision extends JPanel implements ActionListener, 
		InterfaceParserEingabe, InterfaceJFrameAenderung, ComponentListener, 
		InterfaceZoomAenderung{
	
	private static Sprachen	sprachen = Sprachen.holeSprache();
	private static ListResourceBundle	sp = sprachen.holeSprachBundle();
	
	/*Diese Strings werden spaeter ins TextArea ausgegeben.*/
	private String subtrahenten = "";
	private String ergebnisse = "";
	private String restergebnisse = "";
	private String bruchterm;
	private String dividenden;
	private String divisoren;
	
	private String endergebnis = "";
	private int endergebnislaenge;
	
	/*Diese Variablen speichern die Polynome.*/
	double zaehlerzwischenspeicher[];
	private double parametera[];
	private double parameterb[];
	
	/*Diese Varible gibt an, ob die Schrittweise Polynomdivision laeuft.*/
	private boolean eslaeuft;
	
	/*Hier wird der Grad des jeweiligen Polynoms gespeichert.*/
	private int zaehlergrad;
	private int nennergrad;
	private int endergebnisgrad;
	
	/*Diese Booleans geben an, welche der Funktionen gerade angezeigt werden.*/
	private boolean zaehleristangezeigt;
	private boolean nenneristangezeigt;
	private boolean ergebnisistangezeigt;
	
	/*Diese Variable gibt an, ob das Schaubild aktiviert ist.*/
	boolean anzeige;
	
	private JLabel beschreibung[];
	
	/*Die folgenden Variablen dienen der Erstellung der Oberflaeche.*/
	private JPanel eingabepanel;
	private JCheckBox zaehleranzeigen;
	private JCheckBox nenneranzeigen;
	private JCheckBox ergebnisanzeigen;
	private JButton zeichnen;
	private JButton zurueck;
	private JRadioButton gemischt;
	private JRadioButton ohnerest;
	private ButtonGroup gruppea;
	private JRadioButton ganzausrechnen;
	private JRadioButton schrittfuerschritt;
	private ButtonGroup gruppeb;
	private Schaubild schaubild;
	private ZoomBedienung zoom;
	
	private JSplitPane trenner;
	
	private JScrollPane ausgabe;
	private JPanel ausgabepanel;
	private JTextField dividendeingabe;
	private JLabel bruchstrich;
	private JTextField divisoreingabe;
	private JButton ausrechnen;
	private JButton aufgaben;
	private JScrollPane scrollpane;
	private JTextArea textarea;
	
	/*Nun folgen einige Umgebungsvariablen.*/
	private ArrayList collection;
	
	private Random random;
	private Parser parser;
	private Insets insets;
	
	private final int CENTER = GridBagConstraints.CENTER;
	private final int NORTH  = GridBagConstraints.NORTH;
	private final int BOTH = GridBagConstraints.BOTH;
	private final int HORIZONTAL = GridBagConstraints.HORIZONTAL;
	
	/**Der Constructor initialisiert die Polynomdividion.
	 * Im einzelnen wird die Oberflaeche des Programms erstellt und alle
	 * Variablen in ihren Anfangszustand gesetzt.
	 */
	public PolynomDivision(){
		setLayout(new GridLayout());
		insets = new Insets(1,1,1,1);
		
		beschreibung = new JLabel [4];
		for (int i=0;i<4;i++){
			beschreibung[i] = new JLabel("");
			beschreibung[i].setBackground(new Color(100,139,203));
			beschreibung[i].setOpaque(true);
			beschreibung[i].setForeground(new Color(255,255,255));
			beschreibung[i].setHorizontalAlignment(SwingConstants.CENTER);
		}
		
		eingabepanel = new JPanel();
		eingabepanel.setLayout(new GridBagLayout());
			
			beschreibung[0].setText("");
			beschreibung[0].setText(sp.getString("PolyDiv_beschreibung0"));
			GridBagHinzufueger.add(eingabepanel,beschreibung[0],1,1,1,1,0,0,BOTH,CENTER,insets);
			
			ganzausrechnen = new JRadioButton("");
			ganzausrechnen.setText(sp.getString("PolyDiv_ganzausrechnen"));
			GridBagHinzufueger.add(eingabepanel,ganzausrechnen,1,2,1,1,0,0,HORIZONTAL,CENTER,insets);
			ganzausrechnen.setSelected(false);
			
			schrittfuerschritt = new JRadioButton("");
			schrittfuerschritt.setText(sp.getString("PolyDiv_schrittfuerschritt"));
			GridBagHinzufueger.add(eingabepanel,schrittfuerschritt,1,3,1,1,0,0,HORIZONTAL,CENTER,insets);
			schrittfuerschritt.setSelected(true);
		
			gruppeb = new ButtonGroup();
			gruppeb.add(ganzausrechnen);
			gruppeb.add(schrittfuerschritt);
			
			beschreibung[1].setText("");
			beschreibung[1].setText(sp.getString("PolyDiv_beschreibung1"));
			GridBagHinzufueger.add(eingabepanel,beschreibung[1],1,4,1,1,0,0,BOTH,CENTER,insets);
			
			gemischt = new JRadioButton("");
			gemischt.setText(sp.getString("PolyDiv_gemischt"));
			GridBagHinzufueger.add(eingabepanel,gemischt,1,5,1,1,0,0,HORIZONTAL,CENTER,insets);
			gemischt.setSelected(false);
		
			ohnerest = new JRadioButton("");
			ohnerest.setText(sp.getString("PolyDiv_ohnerest"));
			GridBagHinzufueger.add(eingabepanel,ohnerest,1,6,1,1,0,0,HORIZONTAL,CENTER,insets);
			ohnerest.setSelected(true);
		
			gruppea = new ButtonGroup();
			gruppea.add(gemischt);
			gruppea.add(ohnerest);
			
			beschreibung[2].setText("");
			beschreibung[2].setText(sp.getString("PolyDiv_beschreibung2"));
			GridBagHinzufueger.add(eingabepanel,beschreibung[2],1,7,1,1,0,0,BOTH,CENTER,insets);
			
			zaehleranzeigen = new JCheckBox("");
			zaehleranzeigen.setText(sp.getString("PolyDiv_zaehleranzeigen"));
			zaehleranzeigen.setEnabled(false);
			zaehleranzeigen.setBackground(Color.blue);
			GridBagHinzufueger.add(eingabepanel,zaehleranzeigen,1,8,1,1,0,0,BOTH,CENTER,insets);
			
			nenneranzeigen = new JCheckBox("");
			nenneranzeigen.setText(sp.getString("PolyDiv_nenneranzeigen"));
			nenneranzeigen.setEnabled(false);
			nenneranzeigen.setBackground(Color.red);
			GridBagHinzufueger.add(eingabepanel,nenneranzeigen,1,9,1,1,0,0,BOTH,CENTER,insets);
			
			ergebnisanzeigen = new JCheckBox("");
			ergebnisanzeigen.setText(sp.getString("PolyDiv_ergebnisanzeigen"));
			ergebnisanzeigen.setEnabled(false);
			ergebnisanzeigen.setBackground(Color.green);
			GridBagHinzufueger.add(eingabepanel,ergebnisanzeigen,1,10,1,1,0,0,BOTH,CENTER,insets);
			
			zeichnen = new JButton("");
			zeichnen.setText(sp.getString("PolyDiv_zeichnen"));
			zeichnen.setEnabled(false);
			zeichnen.addActionListener(this);
			GridBagHinzufueger.add(eingabepanel,zeichnen,1,11,1,1,0,0,BOTH,CENTER,insets);
			
			zurueck = new JButton("");
			zurueck.setText(sp.getString("PolyDiv_zurueck"));
			zurueck.addActionListener(this);
			zurueck.setEnabled(false);
			GridBagHinzufueger.add(eingabepanel,zurueck,1,12,1,1,0,0,BOTH,CENTER,insets);
			
			schaubild = new Schaubild(this);
			schaubild.addComponentListener(this);
			zoom = new ZoomBedienung(schaubild.holeVergroesserungsStufeX(),schaubild.holeVergroesserungsStufeY(), this);
			zoom.setEnabled(false);
			GridBagHinzufueger.add(eingabepanel,zoom,1,14,1,1,0,0,BOTH,CENTER,insets);
		
		ausgabepanel = new JPanel();
		ausgabepanel.setLayout(new GridBagLayout());
		ausgabe = new JScrollPane(ausgabepanel);
		
			dividendeingabe = new JTextField();
			dividendeingabe.addActionListener(this);
			dividendeingabe.setActionCommand("dividendeingabe");
			GridBagHinzufueger.add(ausgabepanel,dividendeingabe,1,1,15,1,0,0,HORIZONTAL,CENTER,insets);
		
			bruchstrich = new JLabel("----------------------------------------------  =");
			bruchstrich.setFont(new Font("Courier New",Font.BOLD,16));
			GridBagHinzufueger.add(ausgabepanel,bruchstrich,1,2,15,1,0,0,HORIZONTAL,CENTER,insets);
		
			divisoreingabe = new JTextField();
			divisoreingabe.addActionListener(this);
			divisoreingabe.setActionCommand("divisoreingabe");
			GridBagHinzufueger.add(ausgabepanel,divisoreingabe,1,3,15,1,0,0,HORIZONTAL,CENTER,insets);
			
			ausrechnen = new JButton("");
			ausrechnen.setText(sp.getString("PolyDiv_ausrechnen"));
			ausrechnen.addActionListener(this);
			GridBagHinzufueger.add(ausgabepanel,ausrechnen,1,4,1,1,0,0,HORIZONTAL,CENTER,insets);
			
			aufgaben = new JButton("");
			aufgaben.setText(sp.getString("PolyDiv_aufgaben"));
			aufgaben.addActionListener(this);
			GridBagHinzufueger.add(ausgabepanel,aufgaben,2,4,1,1,0,0,HORIZONTAL,CENTER,insets);
			
			textarea = new JTextArea(15,8);
			textarea.setEditable(false);
			textarea.setFont(new Font("Courier New",Font.PLAIN,12));
			scrollpane = new JScrollPane(textarea);
			GridBagHinzufueger.add(ausgabepanel,scrollpane,1,5,15,10,0,0,BOTH,CENTER,insets);
		
		trenner = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true,eingabepanel,ausgabepanel);
		add(trenner);
		
		collection = new ArrayList();
		anzeige = false;
		
		zaehleristangezeigt = false;
		nenneristangezeigt = false;
		ergebnisistangezeigt = false;
		
		eslaeuft = false;
		
		parametera = new double[11];
		parameterb = new double[11];
		
		random = new Random();
		parser = Parser.getInstance();
		
	}
	
	/**Mit dieser Methode werden die Werte aus den Eingabefeldern ausgelesen.
	 */
	private void holeWerte(){
		parametera = parser.parameterDesPolynoms(dividendeingabe.getText(),this);
		zaehlerzwischenspeicher = parametera;
		if(Parser.aktualisierenErlaubt){
			parameterb = parser.parameterDesPolynoms(divisoreingabe.getText(),this);
		
			if(parametera == null && parameterb == null){
				textarea.setText(sp.getString("PolyDiv_dividend"));
				textarea.append("\n"+sp.getString("PolyDiv_divisor"));
			}
			else if(parametera == null){
				textarea.setText(sp.getString("PolyDiv_dividend"));
			}
			else if(parameterb == null){
				textarea.setText(sp.getString("PolyDiv_divisor"));
			}
			else{
				
				/*Dieser algorythmus tritt oefter auf. Er soll den Grad der jeweiligen Funktion bestimmen.*/
				zaehlergrad = 10;
				nennergrad = 10;
				boolean ok = false;
				while (ok == false){
					
					if(parametera[zaehlergrad] == 0.0){
						if(zaehlergrad>0){
							zaehlergrad--;
						}
						else if(zaehlergrad == 0){
							ok = true;
						}
					}
					else if(parameterb[nennergrad] == 0.0){
						if(nennergrad>0){
							nennergrad--;
						}
						else if(nennergrad == 0){
							ok = true;
						}
						
					}
					else if(parameterb[nennergrad] != 0.0){
						ok = true;
					}
					else if(parametera[zaehlergrad] != 0.0){
						ok = true;
					}
				}
				endergebnisgrad = zaehlergrad - nennergrad;
			}
		}
	}
	
	/**Diese Methode gibt immer wenn noetig den Zaehler und Nenner in der obersten Zeile
	 * der TextArea aus.
	 */
	private void aufgabenausgabe(){
		divisoren = "";
		if(zaehlergrad>1){
			bruchterm = "("+runden(parametera[zaehlergrad])+"x^"+zaehlergrad;
			dividenden = "("+runden(parametera[zaehlergrad])+"x^"+zaehlergrad;
		}
		else if(zaehlergrad==1){
			bruchterm = "("+runden(parametera[zaehlergrad])+"x";
			dividenden = "("+runden(parametera[zaehlergrad])+"x";
		}
		else if(zaehlergrad==0){
			bruchterm = "("+runden(parametera[zaehlergrad]);
			dividenden = "("+runden(parametera[zaehlergrad]);
		}
		for(int i=(zaehlergrad-1);i>-1;i--){
			bruchterm = anhaengen(bruchterm,i,zaehlergrad,parametera[i],true);
			dividenden = anhaengen(bruchterm,i,zaehlergrad,parametera[i],true);
		}
		dividenden = bruchterm + ")";
		bruchterm = bruchterm + "):(";
		
		for(int i=nennergrad;i>-1;i--){
			if(i == nennergrad){
				if(i>1){
					divisoren = divisoren + parameterb[i] + "x^" + i;
					bruchterm = bruchterm + parameterb[i] + "x^" + i;
				}
				else if(i==1){
					divisoren = divisoren + parameterb[i] + "x";
					bruchterm = bruchterm + parameterb[i] + "x";
				}
				else if(i==0){
					divisoren = divisoren + parameterb[i];
					bruchterm = bruchterm + parameterb[i];
				}
			}
			else if(i != nennergrad){
				divisoren = anhaengen(divisoren,i,nennergrad,parameterb[i],false);
				bruchterm = anhaengen(bruchterm,i,nennergrad,parameterb[i],true);
			}
		}
		bruchterm = bruchterm + ")=";
		
		endergebnis="";
		
		textarea.append(bruchterm+endergebnis);
		textarea.append("\n");
		
	
	}
	
	/**Mit dieser Methode wird die Polynomdivision einmal von anfang bis ende
	 * durchgefuehrt und das ergebnis ausgegeben.
	 */
	private void polydivdurchstarten(){
		double zwischenspeichera[] = parametera;
		double zwischenspeicherb[] = parametera;
		
		int zaehlera = zaehlergrad;
		if(zaehlergrad >= nennergrad){
			while((zaehlera-nennergrad)>-1){
				zwischenspeichera = polydivision(zwischenspeicherb, parameterb);
				zwischenspeicherb = zwischenspeichera;
			
				zaehlera = 10;
			
				boolean ok = false;
				while (ok == false){
					if(zwischenspeicherb[zaehlera] == 0.0){
						if(zaehlera>0){
							zaehlera--;
						}
						else if(zaehlera == 0){
							ok = true;
							zaehlera = -1;
						}
					}
					else if(zwischenspeicherb[zaehlera] != 0.0){
						ok = true;
					}
				}
			}
			endergebnislaenge = endergebnis.length();
			
			if(zaehlera > -1){
				endergebnis = endergebnis + "+("+restergebnisse+")"+"/("+divisoren+")";
				textarea.replaceRange(endergebnis, bruchterm.length(), (bruchterm.length()+ endergebnislaenge));
			}
		}
		else if(zaehlergrad < nennergrad){
			textarea.setText(sp.getString("PolyDiv_grad"));
		}
		
		eslaeuft = false;
		dividendeingabe.setEnabled(true);
		divisoreingabe.setEnabled(true);
	}
	
	/**Diese Methode kann die Polynomdivision Schrittweise durchfuehren.
	 */
	private void polydivlangsamstarten(){
		double zwischenspeicher[] = zaehlerzwischenspeicher;
		
		if(zaehlergrad<nennergrad && !eslaeuft){
			textarea.setText(sp.getString("PolyDiv_grad"));
			
			eslaeuft = false;
			dividendeingabe.setEnabled(true);
			divisoreingabe.setEnabled(true);
			
			zaehleranzeigen.setEnabled(true);
			nenneranzeigen.setEnabled(true);
			ergebnisanzeigen.setEnabled(true);
			zeichnen.setEnabled(true);
		}
		else if((zaehlergrad-nennergrad)>=0){
			zwischenspeicher = polydivision(zaehlerzwischenspeicher, parameterb);
			zaehlerzwischenspeicher = zwischenspeicher;
			
			zaehlergrad = 10;
			
			boolean ok = false;
			while (ok == false){
				if(zaehlerzwischenspeicher[zaehlergrad] == 0.0){
					if(zaehlergrad>0){
						zaehlergrad--;
					}
					else if(zaehlergrad == 0){
						ok = true;
						zaehlergrad = -1;
					}
				}
				else if(zaehlerzwischenspeicher[zaehlergrad] != 0.0){
					ok = true;
				}
			}
			if((zaehlergrad-nennergrad)<0){
				eslaeuft = false;
				dividendeingabe.setEnabled(true);
				divisoreingabe.setEnabled(true);
				zaehleranzeigen.setEnabled(true);
				nenneranzeigen.setEnabled(true);
				ergebnisanzeigen.setEnabled(true);
				zeichnen.setEnabled(true);
				
				if(zaehlergrad > -1){
					endergebnislaenge = endergebnis.length();
					endergebnis = endergebnis + "+("+restergebnisse+")"+"/("+divisoren+")";
					textarea.replaceRange(endergebnis, bruchterm.length(), (bruchterm.length()+ endergebnislaenge));
					zaehlergrad = -1;
				}
				
			}
			else{
				eslaeuft = true;
				dividendeingabe.setEnabled(false);
				divisoreingabe.setEnabled(false);
				zaehleranzeigen.setEnabled(false);
				nenneranzeigen.setEnabled(false);
				ergebnisanzeigen.setEnabled(false);
				zeichnen.setEnabled(false);
			}
		}
		else if((zaehlergrad-nennergrad)<0){
			eslaeuft = false;
			dividendeingabe.setEnabled(true);
			divisoreingabe.setEnabled(true);
			zaehleranzeigen.setEnabled(true);
			nenneranzeigen.setEnabled(true);
			ergebnisanzeigen.setEnabled(true);
			zeichnen.setEnabled(true);
			
			if(zaehlergrad > -1){
				endergebnislaenge = endergebnis.length();
				endergebnis = endergebnis + "+("+restergebnisse+")"+"/("+divisoren+")";
				textarea.replaceRange(endergebnis, bruchterm.length(), (bruchterm.length()+ endergebnislaenge));
				zaehlergrad = -1;
			}
		}
	}
	
	/**Bei dieser Methode handelt es sich um das Hetrzstueck des Programms.
	 * Hier werden die einzelnen Werte ausgerechnet.
	 */
	private double[] polydivision(double[] zaehler, double[] nenner){
		
		double zaehlerneu []= new double[11];
		double subtrahent[] = new double[11];
		
		subtrahenten = "";
		ergebnisse = "";
		restergebnisse = "";
		
		int position = 0;
		
		zaehlergrad = 10;
		nennergrad = 10;
		boolean ok = false;
		while (ok == false){
			if(zaehler[zaehlergrad] == 0.0){
				if(zaehlergrad>0){
					zaehlergrad--;
				}
				else if(zaehlergrad == 0){
					ok = true;
				}
			}
			else if(nenner[nennergrad] == 0.0){
				if(nennergrad>0){
					nennergrad--;
				}
				else if(nennergrad == 0){
					ok = true;
				}
			}
			else if(zaehler[zaehlergrad] != 0.0){
				ok = true;
			}
		}
		
		
		double zwischenspeicher = zaehler[zaehlergrad]/nenner[nennergrad];
		
		endergebnislaenge = endergebnis.length();
		if(endergebnisgrad != (zaehlergrad-nennergrad)){
			endergebnis = anhaengen(endergebnis,(zaehlergrad-nennergrad),endergebnisgrad,zwischenspeicher,true);
		}
		else if(endergebnisgrad == (zaehlergrad-nennergrad)){
			if((zaehlergrad-nennergrad)>1){
				endergebnis = endergebnis + runden(zwischenspeicher) +"x^"+(zaehlergrad-nennergrad);
			}
			else if((zaehlergrad-nennergrad)==1){
				endergebnis = endergebnis + runden(zwischenspeicher) +"x";
			}
			else if ((zaehlergrad-nennergrad)==0){
				endergebnis = endergebnis + runden(zwischenspeicher);
			}
		}
		
		for(int i=0;i<11;i++){
			zaehlerneu[i] = 0.0;
			subtrahent[i] = 0.0;
		}
		
		for(int i=nennergrad;i>-1;i--){
			if(i+(zaehlergrad-nennergrad)>-1){
				subtrahent[i+(zaehlergrad-nennergrad)] = nenner[i]*zwischenspeicher;
			}
		}
		
		for(int i=zaehlergrad;i>-1;i--){
			
			if(subtrahent[i]!=0){
				zaehlerneu[i] = (zaehler[i] - subtrahent[i]);
			}
			else if(subtrahent[i]==0){
				zaehlerneu[i] = zaehler[i];
			}
			
				position = 0;
				int laenge = 0;
				
				if(i>1){
					position = dividenden.indexOf(""+runden(parametera[i])+"x^"+i,0);
					
						laenge = position-subtrahenten.length();
						for(int k=0;k<laenge-1;k++){
							subtrahenten = subtrahenten +" ";
						}
						subtrahenten = anhaengen(subtrahenten,i,zaehlergrad,subtrahent[i],true);
						
						laenge = position-ergebnisse.length();
						for(int k=0;k<laenge-1;k++){
							ergebnisse = ergebnisse +" ";
						}
						ergebnisse = anhaengen(ergebnisse,i,zaehlergrad,zaehlerneu[i],true);
						if(i == zaehlergrad){
							restergebnisse = restergebnisse +runden(zaehlerneu[i])+"x^"+i;
						}
						else if(i!=zaehlergrad){
							restergebnisse = anhaengen(restergebnisse,i,zaehlergrad,zaehlerneu[i],true);
						}
				}
				else if(i==1){
					position = dividenden.indexOf(""+runden(parametera[i])+"x",0);
					
						laenge = position-subtrahenten.length();
						for(int k=0;k<laenge-1;k++){
							subtrahenten = subtrahenten +" ";
						}
						subtrahenten = anhaengen(subtrahenten,i,zaehlergrad,subtrahent[i],true);
					
						laenge = position-ergebnisse.length();
						for(int k=0;k<laenge-1;k++){
							ergebnisse = ergebnisse +" ";
						}
						ergebnisse = anhaengen(ergebnisse,i,zaehlergrad,zaehlerneu[i],true);
						if(i == zaehlergrad){
							restergebnisse = restergebnisse +runden(zaehlerneu[i])+"x";
						}
						else if(i!=zaehlergrad){
							restergebnisse = anhaengen(restergebnisse,i,zaehlergrad,zaehlerneu[i],true);
						}
				}
				else if(i==0){
					position = dividenden.indexOf(""+runden(parametera[i]),0);
					
						laenge = position-subtrahenten.length();
						for(int k=0;k<laenge-1;k++){
							subtrahenten = subtrahenten +" ";
						}
						subtrahenten = anhaengen(subtrahenten,i,zaehlergrad,subtrahent[i],true);
					
						laenge = position-ergebnisse.length();
						for(int k=0;k<laenge-1;k++){
							ergebnisse = ergebnisse +" ";
						}
						ergebnisse = anhaengen(ergebnisse,i,zaehlergrad,zaehlerneu[i],true);
						if(i == zaehlergrad){
							restergebnisse = restergebnisse +runden(zaehlerneu[i]);
						}
						else if(i!=zaehlergrad){
							restergebnisse = anhaengen(restergebnisse,i,zaehlergrad,zaehlerneu[i],true);
						}
				}
		}
		
		
		textarea.append(subtrahenten);
		textarea.append("\n");
		if((zaehlergrad)>-1){
			int pos = (dividenden.indexOf(runden(parametera[zaehlergrad])+"x^"+(zaehlergrad),0))-2;
			//																				^damits besser aussieht!!
			for(int i=0;i<pos;i++){
				textarea.append(" ");
			}
			int laenge = ergebnisse.length()-pos;
			
			for(int i=0;i<(laenge)+1;i++){
				//				   ^damits besser aussieht!!
				textarea.append("-");
			}
			textarea.append("\n");
			textarea.append(ergebnisse);
		}
		textarea.append("\n");
		
		textarea.replaceRange(endergebnis, bruchterm.length(), (bruchterm.length()+ endergebnislaenge));

		return zaehlerneu;
	}
	
	/**Diese Methode befasst sich damit die einzelnen Strings zu verleangern.
	 * Abhaengig vom Exponenten und Vorzeichen des x werden die einzelnen Werte angehaengt.
	 * Mit Hilfe von Leerstellen werden sie an die richtige Position gesetzt.
	 */
	private String anhaengen(String string,int anhaengergrad, int maxgrad, double wert, boolean nullen){
		String zwischenspeicher = string;
		if(anhaengergrad == maxgrad){
			if(wert > 0.0){
				zwischenspeicher = zwischenspeicher +" "+runden(wert) + "x^"+ anhaengergrad;
			}
			else if(wert == 0.0 && nullen == true){
				zwischenspeicher = zwischenspeicher + " 0.0x^"+ anhaengergrad;
			}
			else if(wert < 0){
				zwischenspeicher = zwischenspeicher + "-" + runden(wert*(-1)) + "x^"+ anhaengergrad;
			}
		}
		else if(anhaengergrad > 1){
			if(wert > 0.0){
				zwischenspeicher = zwischenspeicher + "+" + runden(wert) + "x^"+ anhaengergrad;
			}
			else if(wert == 0.0 && nullen == true){
				zwischenspeicher = zwischenspeicher + "+" + "0.0x^"+ anhaengergrad;
			}
			else if(wert < 0){
				zwischenspeicher = zwischenspeicher + "-" + runden(wert*(-1)) + "x^"+ anhaengergrad;
			}
		}
		else if(anhaengergrad == 1){
			if(wert > 0.0){
				zwischenspeicher = zwischenspeicher + "+" + runden(wert) + "x";
			}
			else if(wert == 0.0 && nullen == true){
				zwischenspeicher = zwischenspeicher + "+" + "0.0x";
			}
			else if(wert < 0){
				zwischenspeicher = zwischenspeicher + "-" + runden(wert*(-1)) + "x";
			}
		}
		else if(anhaengergrad == 0){
			if(wert > 0.0){
				zwischenspeicher = zwischenspeicher + "+" + runden(wert);
			}
			else if(wert == 0.0 && nullen == true){
				zwischenspeicher = zwischenspeicher + "+" + "0.0";
			}
			else if(wert < 0){
				zwischenspeicher = zwischenspeicher + "-" + runden(wert*(-1));
			}
		}
		return zwischenspeicher;
	}
	
	/**Hier wird der Wert einfach gerundet. Diese Methode ist nur bei der Ausgabe
	 * noetig, da ansonsten die Ergebnisse verfaelscht wuerden.
	 */
	private double runden(double wert){
		double zwischenspeichera = (100*wert)+0.5;
		int zwischenspeicherb = (int) zwischenspeichera;
		zwischenspeichera = ((double) zwischenspeicherb)/100;
		return zwischenspeichera;
	}
	
	/**Hier werden einfach Funktionsterme erzeugt.
	 * Um keinen Rest zu erhalten wird das Ergebnis und der Nenner per zufall erzeugt
	 * und dann der Zaehler durch Multiplikation ausgerechnet.
	 * Wenn man einen Rest will rezeugt man einfach Zaehler und Nenner durch Zufall.
	 */
	private void generator(boolean rest){
		
		double dividend[] = new double[11];
		double divisor[]  = new double[11];
		double ergebnis[] = new double[11];
		
		zaehlergrad = 8;
		nennergrad  = 5;
		endergebnisgrad = 3;
		
		if(rest == false){
			for(int i=0;i<nennergrad+1; i++){
				divisor[i] = random.nextInt(6);
			}
			for(int i=0;i<endergebnisgrad+1; i++){
				ergebnis[i] = random.nextInt(6);
			}
			for(int i=0;i<(nennergrad+endergebnisgrad)+1; i++){
				for(int j=0;j<(nennergrad+1); j++){
						for(int k=0;k<(endergebnisgrad+1); k++){
							if((j+k)==i){
								dividend[i] = dividend[i] + divisor[j]*ergebnis[k];
							}
						}
				}
			}
		}
		else if(rest == true){
			for(int i=0;i<zaehlergrad+1; i++){
				dividend[i] = random.nextInt(10);
			}
			for(int i=0;i<nennergrad+1; i++){
				divisor[i] = random.nextInt(10);
			}
		}
		
		
		String textfelda = "";
		String textfeldb = "";
		
		boolean ok = false;
		while (ok == false){
			if(dividend[zaehlergrad] == 0.0){
				if(zaehlergrad>0){
					zaehlergrad--;
				}
				else if(zaehlergrad == 0){
					ok = true;
				}
			}
			else if(divisor[nennergrad] == 0.0){
				if(nennergrad>0){
					nennergrad--;
				}
				else if(nennergrad == 0){
					ok = true;
				}
				
			}
			else if(divisor[nennergrad] != 0.0){
				ok = true;
			}
			else if(dividend[zaehlergrad] != 0.0){
				ok = true;
			}
		}
		endergebnisgrad = zaehlergrad-nennergrad;
		
		for(int i=10;i>-1;i--){
			parametera[i] = dividend[i];
			textfelda = anhaengen(textfelda,i,zaehlergrad,parametera[i],false);
			
			parameterb[i] = divisor[i];
			textfeldb = anhaengen(textfeldb,i,nennergrad,parameterb[i],false);
		}
		
		dividendeingabe.setText(""+textfelda);
		divisoreingabe.setText(""+textfeldb);
		
	}
	

	
	public void actionPerformed(ActionEvent e){
		String s = e.getActionCommand();
		if(s.equals(sp.getString("PolyDiv_ausrechnen"))){
			if(ganzausrechnen.isSelected()==true){
				textarea.setText("");
				holeWerte();
				if(Parser.aktualisierenErlaubt && parametera != null && parameterb != null){
					aufgabenausgabe();
					polydivdurchstarten();
					eslaeuft = false;
					dividendeingabe.setEnabled(true);
					divisoreingabe.setEnabled(true);
					
					zaehleranzeigen.setEnabled(true);
					nenneranzeigen.setEnabled(true);
					ergebnisanzeigen.setEnabled(true);
					zeichnen.setEnabled(true);
				}
			}
			else if(schrittfuerschritt.isSelected()==true){
				if(!eslaeuft){
					textarea.setText("");
					holeWerte();
					if(Parser.aktualisierenErlaubt && parametera != null && parameterb != null){
						aufgabenausgabe();
					}
				}
				if(Parser.aktualisierenErlaubt && parametera != null && parameterb != null){
					polydivlangsamstarten();
				}
			}
		}
		else if(s.equals(sp.getString("PolyDiv_zeichnen"))){
			zurueck.setEnabled(true);
			
			if(!anzeige){
				if(zaehleranzeigen.isSelected()){
					collection.add(new Funktion(dividendeingabe.getText(),false,true,Color.blue,new ArrayList()));
					schaubild.gebeFunktionen(collection);
					
					zaehleristangezeigt = true;
				}
				if(nenneranzeigen.isSelected()){
					collection.add(new Funktion(divisoreingabe.getText(),false,true,Color.red,new ArrayList()));
					schaubild.gebeFunktionen(collection);
					
					nenneristangezeigt = true;
				}
				if(ergebnisanzeigen.isSelected()){
					collection.add(new Funktion(endergebnis,false,true,Color.green,new ArrayList()));
					schaubild.gebeFunktionen(collection);
					
					ergebnisistangezeigt = true;
				}
				
				anzeige = true;
			}
			else if(anzeige){
				if(zaehleranzeigen.isSelected()==false){
					if(zaehleristangezeigt){
						try{
							schaubild.loescheFunktion(dividendeingabe.getText());
						}
						catch(ConcurrentModificationException exception){
							schaubild.loescheFunktion(dividendeingabe.getText());
						}
						
						zaehleristangezeigt = false;
					}
				}
				else if(zaehleranzeigen.isSelected() && !zaehleristangezeigt){
					collection.add(new Funktion(dividendeingabe.getText(),false,true,Color.blue,new ArrayList()));
					schaubild.gebeFunktionen(collection);
					
					zaehleristangezeigt = true;
				}
				if(nenneranzeigen.isSelected()==false){
					if(nenneristangezeigt){
						try{
							schaubild.loescheFunktion(divisoreingabe.getText());
						}
						catch(ConcurrentModificationException exception){
							schaubild.loescheFunktion(divisoreingabe.getText());
						}
						
						nenneristangezeigt = false;
					}
				}
				else if(nenneranzeigen.isSelected() && !nenneristangezeigt){
					collection.add(new Funktion(divisoreingabe.getText(),false,true,Color.red,new ArrayList()));
					schaubild.gebeFunktionen(collection);
					
					nenneristangezeigt = true;
				}
				if(ergebnisanzeigen.isSelected()==false){
					if(ergebnisistangezeigt){
						try{
							schaubild.loescheFunktion(endergebnis);
						}
						catch(ConcurrentModificationException exception){
							schaubild.loescheFunktion(endergebnis);
						}
						
						ergebnisistangezeigt = false;
					}
				}
				else if(ergebnisanzeigen.isSelected() && !ergebnisistangezeigt){
					collection.add(new Funktion(endergebnis,false,true,Color.green,new ArrayList()));
					schaubild.gebeFunktionen(collection);
					
					ergebnisistangezeigt = true;
				}
			}
			
			trenner.setRightComponent(schaubild);
		}
		else if(s.equals(sp.getString("PolyDiv_zurueck"))){
			trenner.setRightComponent(ausgabepanel);
			zurueck.setEnabled(false);
			
			anzeige = false;
		}
		else if(s.equals(sp.getString("PolyDiv_aufgaben"))){
			if(gemischt.isSelected()==true){
				generator(true);
			}
			else if(ohnerest.isSelected()==true){
				generator(false);
			}
			else if(gemischt.isSelected()==false && ohnerest.isSelected()==false){
				ohnerest.setSelected(true);
				generator(false);
			}
			
			eslaeuft = false;
			dividendeingabe.setEnabled(true);
			divisoreingabe.setEnabled(true);
			
			zaehleranzeigen.setEnabled(true);
			nenneranzeigen.setEnabled(true);
			ergebnisanzeigen.setEnabled(true);
			zeichnen.setEnabled(true);
		}
		else if(s.equals("dividendeingabe")){
			divisoreingabe.requestFocus();
		}
		else if(s.equals("divisoreingabe")){
			eslaeuft = false;
			dividendeingabe.setEnabled(true);
			divisoreingabe.setEnabled(true);
			
			zaehleranzeigen.setEnabled(true);
			nenneranzeigen.setEnabled(true);
			ergebnisanzeigen.setEnabled(true);
			zeichnen.setEnabled(true);
			
			if(ganzausrechnen.isSelected()==true){
				textarea.setText("");
				holeWerte();
				if(Parser.aktualisierenErlaubt && parametera != null && parameterb != null){
					aufgabenausgabe();
					polydivdurchstarten();
				}
			}
			else if(schrittfuerschritt.isSelected()==true){
				if(!eslaeuft){
					textarea.setText("");
					holeWerte();
					if(Parser.aktualisierenErlaubt && parametera != null && parameterb != null){
						aufgabenausgabe();
					}
				}
				if(Parser.aktualisierenErlaubt && parametera != null && parameterb != null){
					polydivlangsamstarten();
				}
			}
		}
	}
	
	public void setzeTermEingabe(String term) {}
	public double[] holeTermParameter() {return null;}
	public void setzeTermInformation(String info) {}

	public void funktionFehlerhaft(String fehlerhafterTerm, String korrigierterTerm) {
		if(dividendeingabe.getText().equals(fehlerhafterTerm)){
			dividendeingabe.setText(korrigierterTerm);
			holeWerte();
		}
		else if(divisoreingabe.getText().equals(fehlerhafterTerm)){
			divisoreingabe.setText(korrigierterTerm);
			holeWerte();
		}
	}
	
	public void aktualisiere(){
		if(Parser.aktualisierenErlaubt){
			schaubild.zeichne();
			schaubild.aktualisiere();
		}
	}
	public void componentResized(ComponentEvent arg0) {
		if(anzeige){
			aktualisiere();
		}
	}
	public void componentMoved(ComponentEvent arg0) {
		if(anzeige){
			aktualisiere();
		}
	}
	public void componentShown(ComponentEvent arg0) {
		if(anzeige){
			aktualisiere();
		}
	}
	public void componentHidden(ComponentEvent arg0) {}
	
	public void zoomWurdeVeraendert(ZoomEvent e) {
		if(anzeige){
			schaubild.setzeVergroesserungsStufeX (e.holeXzoom());
			schaubild.setzeVergroesserungsStufeY (e.holeYzoom());
			aktualisiere();
		}
	}
}