package grada;

import grada.geometriefiguren.Funktion;
import grada.geometriefiguren.Punkt;
import grada.interfaces.InterfaceJFrameAenderung;
import grada.interfaces.InterfaceParserEingabe;
import grada.steuereinheit.*;

import java.awt.*;
import java.awt.event.*;
import java.util.*;

import javax.swing.*;

import parser.Parser;
import sprachen.Sprachen;

/**
 * @author Felix
 * @version 2.0 (vom 11.09.2005)
 */
/**Diese Klasse enthaelt das Spiel PunkteTreffen. Mit ihm hat man die 
 * moeglichkeit den umgang mit Graphen zu ueben.
 * Die Aufgabe fuer den Spieler ist es, mit moeglichst wenigen versuchen alle 
 * Ballons zu treffen. Ziel des Spiels ist es alle Ballons getroffen zu haben.
 * Unter anderem ist es moeglich mit mehreren Spielern gegeneinander anzutreten.
 * Die besten Ergebnisse werden am Ende in einer BEstenliste gespeichert.
 */
public class PunkteTreffen extends JPanel implements ActionListener,
			ComponentListener ,InterfaceParserEingabe, InterfaceJFrameAenderung{

	private static Sprachen	sprachen = Sprachen.holeSprache();
	private static ListResourceBundle	sp = sprachen.holeSprachBundle();

	/** Diese Variablen geben den Schwierigkeitsgrad an.*/
	public static final int EINFACH = 1;
	public static final int MITTEL  = 2;
	public static final int SCHWER  = 3;
	
	/**Abhaengig vom Schwierigkeitsgrad werden diese Variablen am Anfang des 
	 * Spiels gesetzt.*/
	/**Hier wird festgehalten, wie viele Punkte ein Spieler p Treffer erhlt.*/
	private int punkteprotreffer;
	/**Hier wird die Anzahl der Ballons festgesetzt.*/
	private int anzahlballons;
	/**Als letztes wird die groesse der Ballons in Einheiten angegeben.*/
	private double ballongroesse;
	
	/** Es folgt das Hauptpanel mit Zubehoer.*/
	private JPanel eingabepanel;
	private JScrollPane scrollpane;
	private JTextField eingabe;
	private JButton zeichnen;
	private JButton neu;
	/** Und ein Panel, was der Ausgabe dient.*/
	private JPanel ausgabepanel;
	private JTable ergebnisse;
	private JScrollPane scrollergebnisse;
	
	/** Als naechstes ist ein Trenner und das Schaubild an der Reihe.*/
	private JSplitPane trenner;
	public Schaubild schaubild;
	
	/** dienen an verschiedenen Stellen der Erlaeuterung der Aufgaben.*/
	private JLabel beschreibung[];
	
	/** Variablen, die zur Oberflchenerstellung notwendig sind.*/
	private Insets insets;
	/**Hier werden die einzelnen Namen oder Punkte angezeigt.*/
	private Object zeilen[][];
	private final int SPALTEN = 3;
	/**es wird gespeichert, wie d. einzelnen Spalten beschriftet werden.*/
	private Object spalten [];
	
	/*Nun kommen einige Umgebungsvariablen des Spiels.*/
	private final int MAXSPIELER  = 5;
	private final int ANZAHLBESCHREIBUNGEN = 2;
	
	/**Schlielich sind alle anderen Variablen an der Reihe, die wrend des 
	 * Spiels bentigt werden.*/
	/**Diese Variable zeigt an, wie viele Spieler mitspielen.*/
	private int anzahlspieler = 1;
	/**Diese Variable gibt spaeter d. Spieler an, der gerade an der Reihe ist.*/
	private int aktuellerspieler = 1;
	/**Diese Variable speichert die Anzahl der Treffer pro durchgang.*/
	private int anzahltreffer = 0;
	/**Hier wird gezaehlt, wie viele Ballons schon abgeschossen wurden.*/
	private int anzahltrefferinsgesamt = 0;
	/**Diese Variable zeigt an, ob das Spiel schon begonnen hat.*/
	private boolean spiellaeuft = false;
	/**Diese Variable spiechert die Spielernamen und ihre Punkte. In der ertsen 
	 * Spalte werden die Namen und in der zweiten die Punkte gespeichert.*/
	private Spieler spieler [];
	/**Diese Variable speichert alles ntige fr einen Ballon. Ob er schon 
	 * getroffen ist erkennt man an seiner Farbe.*/
	private Punkt ballons[];
	
	/**Der Parser "uebersetzt" die Funktinsterme.*/
	public Parser parser;
	ArrayList collection;
	int anzahlgraphen = 0;
	
	/**Der Dialog stellt Zahl der Spieler und deren Namen fest .*/
	private SpieleDialog dialog;
	
	/**Die folgenden Variablen vereinfachen den Quelltext.*/
	private final int CENTER = GridBagConstraints.CENTER;
	private final int NORTH  = GridBagConstraints.NORTH;
	private final int NONE = GridBagConstraints.NONE;
	private final int HORIZONTAL = GridBagConstraints.HORIZONTAL;
	
	
	/**In dieser Methode wird der Schwierigkeitsgrad festgehalten,
	 * alle Variablen initialisiert und die Oberflaeche erstellt.
	 */
	public PunkteTreffen(int schwierigkeit){
		
		switch(schwierigkeit){
			case 1: anzahlballons    = 20;
					ballongroesse    = 1.0;//Radius in Einheiten
					punkteprotreffer = 1;
				break;
			case 2: anzahlballons    = 10;
					ballongroesse    = 0.5;
					punkteprotreffer = 2;
				break;
			case 3: anzahlballons    = 10;
					ballongroesse    = 0.2;
					punkteprotreffer = 3;
				break;
		}
		init();
		erstelleOberflche();
		
		validate();
	}
	
	/**In dieser Methode werden alle Variablen initialisiert, die oefter 
	 * gebraucht werden.*/
	private void init(){
		insets = new Insets(1,1,1,1);
		
		parser = new Parser();
		collection = new ArrayList();
		
		ballons = new Punkt [anzahlballons];
		
		spalten = new Object [SPALTEN];
		spalten[0] = sp.getString("PuTreff_Namen");
		spalten[1] = sp.getString("PuTreff_Punkte");
		spalten[2] = sp.getString("PuTreff_Verhltnis");
		
		spieler = new Spieler[MAXSPIELER];
		zeilen  = new Object [MAXSPIELER][SPALTEN];
		for(int i=0; i<MAXSPIELER; i++){
			/** Hier kann man spaeter die Namen und die Punkte der Spieler 
			 * lesen.*/
			zeilen [i][0]="";
			spieler[i] = new Spieler(sp.getString("PuTreff_Spieler ")+(i+1));
			zeilen [i][1]="";
		}
	}
	
	/**In dieser Methode wird die gesamte Oberflche von PunkteTreffen erstellt.
	 */
	private void erstelleOberflche(){
		setLayout(new GridLayout());
		
		/*Hier wird erst das Eingabepanel erstellt, das das Hauptpanel ist.
		 * Zu dem Panel wird noch ein ausgabepanel hinzugefuegt, 
		 * damit die Oberflaeche gut eingeteilt ist.
		 * Als letztes wir ein Trenner und daneben das Schaubild hinzugefuegt.
		 */
		eingabepanel = new JPanel();
		eingabepanel.setLayout(new GridBagLayout());
		scrollpane = new JScrollPane(eingabepanel);
		
			eingabe = new JTextField();
			eingabe.addActionListener(this);
			eingabe.setEnabled(false);
			eingabe.setActionCommand(sp.getString("PuTreff_Funktion zeichnen"));
			GridBagHinzufueger.add(eingabepanel,eingabe,1,2,4,1,1,0,HORIZONTAL,CENTER,insets);
			
			zeichnen = new JButton(sp.getString("PuTreff_Funktion zeichnen"));
			zeichnen.addActionListener(this);
			zeichnen.setEnabled(false);
			GridBagHinzufueger.add(eingabepanel,zeichnen,2,3,1,1,1,0,NONE,CENTER,insets);
			
			neu = new JButton(sp.getString("PuTreff_Neu"));
			neu.addActionListener(this);
			GridBagHinzufueger.add(eingabepanel,neu,1,3,1,1,1,0,NONE,CENTER,insets);
		
		ausgabepanel = new JPanel();
		ausgabepanel.setLayout(new GridBagLayout());
		
			ergebnisse = new JTable(zeilen,spalten);
			
			/*Hier wird die spaetere Zeilenfaerbung eingestellt, die bei 
			 * mehreren Spieler noetig ist.*/
			ergebnisse.setSelectionBackground(Color.red);
			ergebnisse.setSelectionForeground(Color.black);
			ergebnisse.setEnabled(false);
			
			scrollergebnisse = new JScrollPane(ergebnisse);
			scrollergebnisse.setPreferredSize(new Dimension(SPALTEN*30, (MAXSPIELER*20)-1));
			GridBagHinzufueger.add(ausgabepanel,scrollergebnisse,    1,2,2,3,1,0,HORIZONTAL,CENTER,insets);
			
		insets.set(20,1,1,1);
		GridBagHinzufueger.add(eingabepanel,ausgabepanel,  1,5,4,5,1,1,HORIZONTAL,NORTH, insets);
		insets.set(1,1,1,1);
		
		schaubild = new Schaubild(this);
		schaubild.addComponentListener(this);
		
		trenner = new JSplitPane(JSplitPane.HORIZONTAL_SPLIT,true,scrollpane,schaubild);
		add(trenner);
		
		/*Zuerst werden die Beschreibungen initialisiert, die spter notwendig sind.*/
		beschreibung = new JLabel [ANZAHLBESCHREIBUNGEN];
		for (int i=0;i<ANZAHLBESCHREIBUNGEN;i++){
			beschreibung[i] = new JLabel("");
			beschreibung[i].setBackground(new Color(100,139,203));
			beschreibung[i].setOpaque(true);
			beschreibung[i].setForeground(new Color(255,255,255));
			beschreibung[i].setHorizontalAlignment(SwingConstants.CENTER);
		}
		beschreibung[0].setText(sp.getString("PuTreff_Funktionseingabe:"));
		GridBagHinzufueger.add(eingabepanel,beschreibung[0],1,1,4,1,1,0,HORIZONTAL,CENTER,insets);
		beschreibung[1].setText(sp.getString("PuTreff_Ergebnisse:"));
		GridBagHinzufueger.add(ausgabepanel,beschreibung[1],1,1,2,1,1,0,HORIZONTAL,CENTER,insets);
	}
	
	/**Diese Methode startet den Dialog, der zur Feststellung der 
	 * Spieleranzahl und -namen benutzt wird.
	 */
	private void starteDialog(){
		eingabe.setEnabled(false);
		zeichnen.setEnabled(false);
		neu.setEnabled(false);
		
		dialog = new SpieleDialog(spieler,this,MAXSPIELER, anzahlspieler);
	}
	/**Diese Methode wird ausgefuert, sobalt der Dialog abgebrochen wird.
	 */
	public void dialogIstAbgebrochen(){
		if(spiellaeuft == false){
			eingabe.setEnabled(false);
			zeichnen.setEnabled(false);
			neu.setEnabled(true);
		}
		else if(spiellaeuft == true){
			eingabe.setEnabled(true);
			zeichnen.setEnabled(true);
			neu.setEnabled(true);
		}
	}
	/**Diese Methode wird ausgefuert, sobalt im Dialog OK gedrueckt wird.
	 */
	public void dialogIstAusgefuehrt(int anzahl, Spieler neuespieler[]){
		eingabe.setEnabled(true);
		zeichnen.setEnabled(true);
		neu.setEnabled(true);
		
		spieler = neuespieler;
		anzahlspieler = anzahl;
		for(int i=0;i<MAXSPIELER;i++){
			ergebnisse.setValueAt("",i,0);
			ergebnisse.setValueAt("",i,1);
			ergebnisse.setValueAt("",i,2);
		}
		for(int i=0;i<anzahl;i++){
			spieler[i].setPunkte(0);
			spieler[i].setAnzahlTreffer(0);
			spieler[i].setAnzahlVersuche(0);
			ergebnisse.setValueAt(spieler[i].getName(),i,0);
		}
		
		starteSpiel();
	}
	
	/**Diese Methode startet schlielich ein neues Spiel.
	 */
	private void starteSpiel(){
		spiellaeuft = true;
		
		collection = new ArrayList();
		anzahltrefferinsgesamt = 0;
		anzahlgraphen = 0;
		eingabe.setText("");
		
		getBallonKoordinaten();
		
		aktualisiere();
		
		ergebnisse.setRowSelectionInterval(0,0);
	}
	
	/**Diese Methode "erfindet" Koordinaten fuer die Ballons, wobei darauf 
	 * geachtet wird, dass die sich nicht ueberschneiden.
	 * Zu diesem Zweck wird auch ein "Sicherheitsabstand" eingefuegt.
	 * 
	 * koordinaten[][0] entspricht den x-Koordinaten
	 * koordinaten[][1] entspricht den y-Koordinaten
	 */
	private void getBallonKoordinaten(){
		Random zufall = new Random();
		double zwischenspeicherx = 0.0;
		double zwischenspeichery = 0.0;
		int k;

		for(int i=0;i<anzahlballons; i++){
			zwischenspeicherx = (Math.random()*(20.0 - ((ballongroesse + 0.3)*2)))-(10.0 - (ballongroesse + 0.3));
			zwischenspeichery = (Math.random()*(20.0 - ((ballongroesse + 0.3)*2)))-(10.0 - (ballongroesse + 0.3));
			k = 0;
			while(k<i){
				/*Es wird zuerts die Differenz der x-Koordinaten*/
				double a = zwischenspeicherx - ballons[k].holeX();
				/*dann die Differenz der y-Koordinaten errechnet.*/
				double b = zwischenspeichery - ballons[k].holeY();
				/*Dann wir der Abstand der Ballons mit dem Satz des Pythagoras errechnet.plus Sicherheitsabstand*/
				double abstand = Math.sqrt((Math.pow(a,2))+Math.pow(b,2))-(ballongroesse);
				if(abstand < (0 + 1.0)){
					zwischenspeicherx = (Math.random()*(20.0 - ((ballongroesse + 0.3)*2)))-(10.0 - (ballongroesse + 0.3));
					zwischenspeichery = (Math.random()*(20.0 - ((ballongroesse + 0.3)*2)))-(10.0 - (ballongroesse + 0.3));
					k = 0;
				}
				else if(abstand > (0 + 1.0)){
					k++;
				}
			}
			ballons[i] = new Punkt(i,zwischenspeicherx,zwischenspeichery,ballongroesse,Color.blue);
			/*Hier werden alle Ballons mit Hilfe der Farbe als noch nicht getroffen markiert.*/
		}
	}
	
	private void zeichneBallons(){
		ArrayList ballonsliste = new ArrayList();
		for(int i=0;i<ballons.length;i++){
			ballonsliste.add(ballons[i]);
		}
			schaubild.gebePunkte(ballonsliste);
			
	}
	
	/**Diese Methode errechnet den Abstand vom Rand jedes Ballons zum Graphen.
	 * Wenn dieser zu gering ist gilt der Ballon als getroffen.
	 * Er wird markiert und als Treffer gezaehlt.
	 */
	private void getTreffer(){
		//Diese Variable gibt spter den Abstand vom mittelpunkt eines Ballons 
		//zu einem Punkt auf dem Graphen an.
		double x;
		boolean letzterballon = false;
		anzahltreffer = 0;
		HashMap hashmap = null;
		hashmap = parser.funktionswerte(eingabe.getText(),-10.0, 0.1, 10.0,this);
		//Diese Schleife soll die x-Achse von links nach rechts ueberqueren.
		for(int i=0; i<hashmap.size(); i++){
			x=-10.0+i*0.1;
			int aktuellerballon = 0;
			while(aktuellerballon!=anzahlballons){
				if(ballons[aktuellerballon].holeFarbe() != Color.red){
					double y=((Double)hashmap.get(new Double(x))).doubleValue();
					double a = x - ballons[aktuellerballon].holeX();
					double b = y - ballons[aktuellerballon].holeY();
					
					double entfernung = Math.sqrt(Math.pow(a,2)+Math.pow(b,2));
					
					if (entfernung <= ballongroesse){
						ballons[aktuellerballon].setzeFarbe(Color.red);
						anzahltreffer++;
						anzahltrefferinsgesamt++;
					}
				}
				aktuellerballon++;
			}
		}
	}
	
	/**Diese Methode zaehlt schlielich die Punkte nach dem zeichnen eines 
	 * Graphen.
	 */
	private void zaehlePunkte(){
		int zwischenspeichera = (spieler[aktuellerspieler-1].getPunkte()).intValue();
		int punkte = zwischenspeichera+(anzahltreffer*punkteprotreffer);
		spieler[aktuellerspieler-1].setPunkte(punkte);
		ergebnisse.setValueAt(""+punkte,aktuellerspieler-1,1);
		
		int zwischenspeicherb = spieler[aktuellerspieler-1].getAnzahlTreffer();
		int trefferinsgesamt = zwischenspeicherb + anzahltreffer;
		spieler[aktuellerspieler-1].setAnzahlTreffer(trefferinsgesamt);
		
		int zwischenspeicherc = spieler[aktuellerspieler-1].getAnzahlVersuche();
		int versucheinsgesamt = zwischenspeicherc + 1;
		spieler[aktuellerspieler-1].setAnzahlVersuche(versucheinsgesamt);
		ergebnisse.setValueAt(""+trefferinsgesamt+"/"+versucheinsgesamt,aktuellerspieler-1,2);
		
		if(anzahltrefferinsgesamt == anzahlballons){
			Spielbeenden();
		}
	}
	
	/**Diese Methode wechselt zum naechsten spieler, damit immer festgestellt 
	 * werden kann, welcher gerade an der Reihe ist.
	 */
	private void setnaechsterSpieler(){
		if(aktuellerspieler+1<=anzahlspieler){
			aktuellerspieler++;
		}
		else{
			aktuellerspieler = 1;
		}
		ergebnisse.setRowSelectionInterval(aktuellerspieler-1,aktuellerspieler-1);
	}
	
	public void aktualisiere(){
		if(Parser.aktualisierenErlaubt){
			schaubild.zeichne();
			schaubild.setzeSpiel(-10.0,10.0,-10.0,10.0);
			schaubild.aktualisiere();
			if(spiellaeuft == true){
				zeichneBallons();
			}
		}
	}
	public void funktionFehlerhaft(String alt, String neu){
		eingabe.setText(neu);
		anzahlgraphen--;
		collection.remove(anzahlgraphen);
		collection.add(anzahlgraphen,new Funktion(neu,false,true,Color.red,new ArrayList()));
		schaubild.gebeFunktionen(collection);
		anzahlgraphen++;
		
		aktualisiere();
		
		getTreffer();
		zaehlePunkte();
		setnaechsterSpieler();
		
		eingabe.requestFocus();
		eingabe.selectAll();
	}
	
	private void Spielbeenden(){
		spiellaeuft = false;
		
		eingabe.setEnabled(false);
		zeichnen.setEnabled(false);
		neu.setEnabled(true);
		
		Bestenliste bestenliste = new Bestenliste(spieler,anzahlspieler);
	}
	
	public void actionPerformed(ActionEvent e) {
		String s = e.getActionCommand();
		if(s.equals(sp.getString("PuTreff_Neu"))){
			starteDialog();
		}
		else if(s.equals(sp.getString("PuTreff_Funktion zeichnen"))){
			parser = Parser.getInstance();
			
			collection.add(anzahlgraphen,new Funktion(eingabe.getText(),false,true,Color.red,new ArrayList()));
			schaubild.gebeFunktionen(collection);
			anzahlgraphen++;
			
			getTreffer();
			
			aktualisiere();
			
			zaehlePunkte();
			setnaechsterSpieler();
			
			eingabe.requestFocus();
			eingabe.selectAll();
		}
	}
	
	public void componentHidden(ComponentEvent e){}
	public void componentMoved(ComponentEvent e){
			aktualisiere();
	}
	public void componentResized(ComponentEvent e){
			aktualisiere();
	}
	public void componentShown(ComponentEvent e){
			aktualisiere();
	}
	
	public void setzeTermEingabe(String term) {eingabe.setText(term);}
	
	public double[] holeTermParameter() {
		double[] zwischenspeicher = new double[26];
		for(int i=0;i<26;i++){
			zwischenspeicher[i]=0.0;
		}
		return zwischenspeicher;
	}
	public void setzeTermInformation(String info) {}
}