package grada.steuereinheit;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import javax.swing.*;

import sprachen.Sprachen;

/**
 * @author Felix
 * @version 1.0 (vom 05.07.2005)
 */
/**Diese Klasse Speichert eine Liste der 10 besten Spieler und zeigt diese an.
 */
public class Bestenliste implements ActionListener{
	private Sprachen	sprachen = Sprachen.holeSprache();
	private ListResourceBundle	sp = sprachen.holeSprachBundle();
	private int anzahlspieler;
	private ArrayList arraylist;
	
	private JDialog dialog;
	private JLabel beschreibung;
	private JTable ergebnisse;
	private JButton ok;
	private JScrollPane scrollergebnisse;
	
	private final int CENTER = GridBagConstraints.CENTER;
	private final int HORIZONTAL = GridBagConstraints.HORIZONTAL;
	
	/**@param spieler[] uebergibt die noetigen Daten ueber jeden Spieler.
	 * 		  anzahl    uebergibt die anzahl der Spieler, die am letzten Spiel
	 * 					beteiligt waren.
	 */
	public Bestenliste(Spieler spieler[], int anzahl){
		arraylist = new ArrayList(15);
		anzahlspieler = anzahl;
		
		for(int i=0;i<anzahlspieler;i++){
			arraylist.add(i,spieler[i]);
		}
		ergaenzeBestenliste();
		setBestenliste();
		erstelleOberflaeche();
	}
	
	/**In dieser Methode wird die Bestenliste aus einer Datei ausgelesen, falls 
	 * diese schon existieren sollte. Danach werden die Spieler, die am letzten 
	 * Spiel teilgenommen haben hinzugefuegt.
	 */
	private void ergaenzeBestenliste(){
		FileInputStream fileinput;
		BufferedInputStream inputbuffer;
		ObjectInputStream leser;
		ArrayList zwischenspeicher;
		
		try{
			fileinput = new FileInputStream(sp.getString("BestList_highscorePfad"));
			leser = new ObjectInputStream(fileinput);
			
			zwischenspeicher = (ArrayList) leser.readObject();
			
			for(int i=0;i<zwischenspeicher.size();i++){
				arraylist.add(anzahlspieler+i,zwischenspeicher.get(i));
			}
		}
		catch(FileNotFoundException e)  {}
		catch(IOException e)			{}
		catch(ClassNotFoundException e) {}
	}
	
	/**Hier wird die nun vorhandene ArrayList sortiert und auf die laenge 10 
	 * gekuerzt. Danach wird sie zum speichern in eine Datei geschrieben. 
	 * Diese Datei wird wenn noetig erzeugt.
	 */
	private void setBestenliste(){
		Collections.sort(arraylist);
		int ende = arraylist.size();
		if(ende > 10){
			for(int i=10;i<ende;i++){
				/*Da die Liste immer einz kuerzer wird und damit die indizes 
				 * hochrutschen muss die nummer zehn oefter geloescht werden.
				 */
				arraylist.remove(10);
			}
		}
		FileOutputStream fileoutput;
		ObjectOutputStream schreiber;
		
		try {
			fileoutput = new FileOutputStream(sp.getString("BestList_highscorePfad"));
			schreiber = new ObjectOutputStream(fileoutput);
			schreiber.writeObject(arraylist);
			schreiber.close();
			}
			catch (FileNotFoundException e) {}
			catch (IOException e){}
	}
	
	/**Hier wird schlielich ein Dialog erzeugt, der die Bestenliste anzeigt.
	 */
	private void erstelleOberflaeche(){
		dialog = new JDialog();
		dialog.setLayout(new GridBagLayout());
		
		beschreibung = new JLabel(sp.getString("BestList_Die Bestenliste:"));
		beschreibung.setBackground(new Color(100,139,203));
		beschreibung.setOpaque(true);
		beschreibung.setForeground(new Color(255,255,255));
		beschreibung.setHorizontalAlignment(SwingConstants.CENTER);
		GridBagHinzufueger.add(dialog,beschreibung,1,1,1,3,1,0,HORIZONTAL,CENTER,new Insets(1,1,1,1));
		
		Object spalten []= new Object [2];
		spalten[0] = sp.getString("BestList_Namen");
		spalten[1] = sp.getString("BestList_Punkte");
		Object zeilen[][] = new Object [10][2];
		for(int i=0; i<arraylist.size(); i++){
			zeilen [i][0] = ((Spieler)arraylist.get(i)).getName();
			zeilen [i][1] = ((Spieler)arraylist.get(i)).getPunkte();
		}
		
		ergebnisse = new JTable(zeilen,spalten);
		for(int i=0; i<arraylist.size(); i++){
			zeilen [i][0] = ((Spieler)arraylist.get(i)).getName();
			zeilen [i][1] = ((Spieler)arraylist.get(i)).getPunkte();
		}
		ergebnisse.setEnabled(false);
		scrollergebnisse = new JScrollPane(ergebnisse);
		scrollergebnisse.setPreferredSize(new Dimension(150, 179));
		GridBagHinzufueger.add(dialog,scrollergebnisse,1,2,3,3,1,0,HORIZONTAL,CENTER,new Insets(1,1,1,1));
		
		ok = new JButton(sp.getString("BestList_OK"));
		ok.addActionListener(this);
		GridBagHinzufueger.add(dialog,ok,1,11,2,3,1,0,HORIZONTAL,CENTER,new Insets(1,1,1,1));
		
		dialog.pack();
		dialog.setVisible(true);
		dialog.setLocationRelativeTo(null);
		dialog.setAlwaysOnTop(true);
		dialog.setResizable(false);
	}
	
	public void actionPerformed(ActionEvent e){
		String s = e.getActionCommand();
		if(s.equals(sp.getString("BestList_OK"))){
			dialog.setVisible(false);
		}
	}
}
