package grada.steuereinheit;

import grada.event.FunktionsParameterEvent;
import grada.geometriefiguren.Funktion;
import grada.interfaces.InterfaceFunktionsParameterAenderung;

import java.awt.*;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.AdjustmentEvent;
import java.awt.event.AdjustmentListener;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.ListResourceBundle;

import javax.swing.*;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollBar;

import sprachen.Sprachen;

/**
 * Dieses Klasse bietet fr alle vorhandenen Funktionen und in diesen 
 * Funktionen vorkommenden Formvariablen eine einfache Einstellmglichkeit
 * per Scrollbalken.
 * der Wertebereich kann erweitert werden und der Wert schnell auf 0 
 * gesetzt werden.
 * @author Thilo Beckmann
 * @version 08.07.05
 */
public class FormVariablenScrollbalken extends JPanel implements ActionListener, AdjustmentListener{
	private Sprachen	sprachen = Sprachen.holeSprache();
	private ListResourceBundle	sp = sprachen.holeSprachBundle();
	private InterfaceFunktionsParameterAenderung listener;
	
	private JLabel		scrTitel		= new JLabel(sp.getString("FuAnz_Formvariable-Einstellung"));
	private Insets		abstand			= new Insets(0,0,0,0);
	/** Gibt die maximale Anzahl an Formvariablen an (hier sind es 26 nmlich von A bis Z) */
	public  static final int MAXSCR = 26; // Code von A (65) bis Z (26 codes weiter)
	private static JScrollBar	scr[]	= new JScrollBar[MAXSCR];
	// Parameter des Scrollbalken
	private JLabel		scrPara[]		= new JLabel[MAXSCR];	
	private JLabel		scrName[]		= new JLabel[MAXSCR];
	//Button zum "0-setzen"
	private JButton		scrToNull[]		= new JButton[MAXSCR];	
	//wird dieses Scrollbalken bentigt?	
	private boolean		scrVorhanden[]	= new boolean[MAXSCR];	
	private int			scrmin[]		= new int[MAXSCR];		
	private int			scrmax[]		= new int[MAXSCR];		
	private int			scrextend[]		= new int[MAXSCR];
	
	//Buttons zum vergrern der Scrollbalken
	private JButton		linksmehr[]		= new JButton[MAXSCR]; 
	//Buttons zum verkleinern der Scrollbalken
	private JButton		linksweniger[]	= new JButton[MAXSCR];
	//Buttons zum vergrern der Scrollbalken
	private JButton		rechtsmehr[]	= new JButton[MAXSCR];
	//Buttons zum verkleinern der Scrollbalken
	private JButton		rechtsweniger[]	= new JButton[MAXSCR];
	
	private ArrayList	fuListe;
	
	/** gibt die Scrollbalken zurck.
	 * 
	 * @return Scrollbalken
	 */
	public static JScrollBar[] holeScrollbalken(){
		return scr;
	}
	
	/** Konstruktor.
	 * hier werden die wichtigsten Werte initialisiert, damit eine korrekte 
	 * Anzeige mchlich ist. Dazu gehren die FunktionsListe und der Listener, 
	 * der bentigt wird, um die nderungen auch an die richtige Stelle 
	 * weiterzuleiten. 
	 * @param fuListe	FunktionsListe in Form einer ArrayList
	 * @param listener	Listener vom Typ InterfaceFunktionsParameterAenderung
	 */
	public FormVariablenScrollbalken(ArrayList fuListe, InterfaceFunktionsParameterAenderung listener){
		this.listener = listener;
		this.fuListe = fuListe;
		Funktion funktion;
		setLayout(new GridBagLayout());
		initialisiere();
		aktualisiere();
		setVisible(true);
	}

	/** initialisiert Werte.
	 * hier werden verschiedene Werte gesetzt, die nur einmal aufgerufen 
	 * werden sollten.
	 */
	private void initialisiere(){
		for (int i=0; i<MAXSCR; i++){
			scrextend[i]	= 50;
			scrmin[i]		= -500;
			scrmax[i]		= scrmin[i]*-1+scrextend[i];
			scrVorhanden[i]	= false;
			linksmehr[i]	= new JButton(new ImageIcon("grafik/icons/+.png"));
			linksweniger[i]	= new JButton(new ImageIcon("grafik/icons/-.png"));
			rechtsmehr[i]	= new JButton(new ImageIcon("grafik/icons/+.png"));
			rechtsweniger[i]= new JButton(new ImageIcon("grafik/icons/-.png"));

		}
	}
	/** aktualisiert die Anzeige.
	 * Alle anzuzeigenden Elemente werden aktualisiert (z.B. wenn sich Werte von
	 * Scrollbalken ndern, oder wenn Formvariablen aus einer oder mehreren 
	 * Funktionen gelscht wurden und somit nicht mehr angezeigt werden mssen. 
	 */
	private void aktualisiere(){
		Funktion funktion;
		for (int i=0; i<MAXSCR; i++){
			scrVorhanden[i]	= false;
		}
		
		// Suche Parameter in FunktionsTermen
		for(int k=65; k<65+MAXSCR; k++){
			scrVorhanden[k-65] = false;
		}
		String funktionsterm = "";
		Iterator iterator = fuListe.iterator();
		while (iterator.hasNext()){
			funktion = (Funktion)iterator.next();
			funktionsterm = funktion.holeFunktionsTerm();
			for (int i=0; i<funktionsterm.length(); i++){
				for(int k=65; k<65+MAXSCR; k++){
					if (funktionsterm.charAt(i)==(char)k){
						scrVorhanden[k-65] = true;
					}
				}
			}
		}
		
		int zeile = 0;
		for (int i=0; i<MAXSCR; i++){
			if (scrVorhanden[i]){
				if (scr[i]==null){
					scrName[i]		= new JLabel(""+(char)(i+65));
					scr[i]			= new JScrollBar (JScrollBar.HORIZONTAL,0, 50, scrmin[i], scrmax[i]);
					scr[i]			.addAdjustmentListener(this);
					scrPara[i]		= new JLabel (""+((double)scr[i].getValue()/100.0));
					scrPara[i]		.setBackground( new Color(200,200,0));
					scrToNull[i]	= new JButton (sp.getString("FuAnz_->0"));
					scrToNull[i]	.setActionCommand("to0;"+i);
					scrToNull[i]	.setToolTipText(sp.getString("FuAnz_diese FormVariable auf 0 (Null) setzen"));
					scrToNull[i]	.setMargin(abstand);
					scrToNull[i]	.addActionListener(this);
					
					linksmehr[i]	= new JButton(new ImageIcon("grafik/icons/+.png"));
					linksweniger[i]	= new JButton(new ImageIcon("grafik/icons/-.png"));
					rechtsmehr[i]	= new JButton(new ImageIcon("grafik/icons/+.png"));
					rechtsweniger[i]= new JButton(new ImageIcon("grafik/icons/-.png"));
					
					linksmehr[i]	.setToolTipText(sp.getString("FuAnz_den Wertebereich dieses Scrollbalkens auf dieser Seite erweitern"));
					linksweniger[i]	.setToolTipText(sp.getString("FuAnz_den Wertebereich dieses Scrollbalkens auf dieser Seite verringern"));
					rechtsmehr[i]	.setToolTipText(sp.getString("FuAnz_den Wertebereich dieses Scrollbalkens auf dieser Seite erweitern"));
					rechtsweniger[i].setToolTipText(sp.getString("FuAnz_den Wertebereich dieses Scrollbalkens auf dieser Seite verringern"));
					
					linksmehr[i]	.setSize(5,5);
					linksweniger[i]	.setSize(5,5);
					rechtsmehr[i]	.setSize(5,5);
					rechtsweniger[i].setSize(5,5);
					
					linksmehr[i]	.setMargin(abstand);
					linksweniger[i]	.setMargin(abstand);
					rechtsmehr[i]	.setMargin(abstand);
					rechtsweniger[i].setMargin(abstand);

					linksmehr[i]	.setActionCommand("lm;"+i);
					linksweniger[i]	.setActionCommand("lw;"+i);
					rechtsmehr[i]	.setActionCommand("rm;"+i);
					rechtsweniger[i].setActionCommand("rw;"+i);
					
					linksmehr[i]	.addActionListener(this);
					linksweniger[i]	.addActionListener(this);
					rechtsmehr[i]	.addActionListener(this);
					rechtsweniger[i].addActionListener(this);
				}
				scr[i].setMinimum(scrmin[i]);
				scr[i].setMaximum(scrmax[i]);

				GridBagHinzufueger.add(this,scrName[i],			0,zeile,	1,2, 0,0,GridBagConstraints.NONE,GridBagConstraints.EAST,abstand);
				GridBagHinzufueger.add(this,linksmehr[i],		1,zeile,	1,1, 0,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
				GridBagHinzufueger.add(this,linksweniger[i],	1,zeile+1,	1,1, 0,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
				GridBagHinzufueger.add(this,scr[i],				2,zeile,	1,2,10,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
				GridBagHinzufueger.add(this,rechtsmehr[i],		3,zeile,	1,1, 0,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
				GridBagHinzufueger.add(this,rechtsweniger[i],	3,zeile+1,	1,1, 0,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
				GridBagHinzufueger.add(this,scrPara[i],			4,zeile,	1,2, 0,0,GridBagConstraints.NONE,GridBagConstraints.EAST,abstand);
				GridBagHinzufueger.add(this,scrToNull[i],		5,zeile,	1,2, 0,0,GridBagConstraints.NONE,GridBagConstraints.EAST,abstand);	
				zeile++;
				zeile++;
			}
			else if (!scrVorhanden[i]){
				if (scr[i]!=null){
					GridBagHinzufueger.remove(this,scrName[i]);
					GridBagHinzufueger.remove(this,scr[i]);
					GridBagHinzufueger.remove(this,scrPara[i]);
					GridBagHinzufueger.remove(this,scrToNull[i]);
					GridBagHinzufueger.remove(this,linksmehr[i]);
					GridBagHinzufueger.remove(this,linksweniger[i]);
					GridBagHinzufueger.remove(this,rechtsmehr[i]);
					GridBagHinzufueger.remove(this,rechtsweniger[i]);
					scrName[i]	= null;
					scr[i]		= null;
					scrPara[i]	= null;
					scrToNull[i]= null;
					linksmehr[i]	= null;
					linksweniger[i]	= null;
					rechtsmehr[i]	= null;
					rechtsweniger[i]= null;
				}
			}
		}
	}
	
	/** setzt die FunktionsListe
	 * wenn von auen aktualisiert werden soll, muss die aktuelle FunktionsListe 
	 * bergeben werden, damit die Scrollbalken sich daran anpassen.
	 * 
	 * @param fuListe	FunktionsListe vom Typ ArrayList
	 */
	public void setzeFunktionsListe(ArrayList fuListe){
		this.fuListe = fuListe;
		aktualisiere();
	}
	
	/** wenn einer der Buttons gedrckt wurde, wird die entsprechende Aktion 
	 * durchgefhrt, bzw. ein Event an den Listener "gesendet"/bergeben.
	 */
	public void actionPerformed(ActionEvent e){
		String cmd = e.getActionCommand();
		String s[] = new String[10];
		String st[] = cmd.split(";");
		
		if (cmd.startsWith("to0;")){
			s = cmd.split(";");
			scr[new Integer(s[1]).intValue()].setValue(0);
			listener.funktionsParameterWurdenVeraendert(new FunktionsParameterEvent(scr));
		}
		else if (cmd.startsWith("lm;")){
			s = cmd.split(";");
			
			scrmin[new Integer(st[1]).intValue() ] -= 500; 
			aktualisiere();
		}
		else if (cmd.startsWith("lw;")){
			s = cmd.split(";");
			System.out.println("ID: "+s[1]);
			if (scrmin[new Integer(st[1]).intValue()]+600 < scrmax[new Integer(st[1]).intValue()]){
				scrmin[new Integer(st[1]).intValue()] +=500;
			}
			aktualisiere();
		}
		else if (cmd.startsWith("rm;")){
			s = cmd.split(";");
			scrmax[new Integer(st[1]).intValue()] += 500;
			aktualisiere();
		}
		else if (cmd.startsWith("rw;")){
			s = cmd.split(";");
			if (scrmax[new Integer(st[1]).intValue()]-600 > scrmin[new Integer(st[1]).intValue()]){
				scrmax[new Integer(st[1]).intValue()] -= 500;
			}
			aktualisiere();
		}
		System.out.println("Command"+cmd);
		
	}
	/** Wenn sich die Werte an den Scrollbalken ndern, muss das
	 * an die listener weitergegeben werden, damit die Anzeige des
	 * Grahphen aktualisiert bzw. gendert wird.
	 */
	public void adjustmentValueChanged(AdjustmentEvent e){
		for (int i=0; i<scr.length; i++){
			if (scrVorhanden[i] && scrPara[i]!=null && scr[i]!=null){
				scrPara[i].setText(""+((double)scr[i].getValue()/100.0));
			}
		}
		listener.funktionsParameterWurdenVeraendert(new FunktionsParameterEvent(scr));
	}
}
