package grada.steuereinheit;

import grada.event.FunktionsEvent;
import grada.event.FunktionsListenEvent;
import grada.geometriefiguren.Funktion;
import grada.interfaces.InterfaceFunktionsAenderung;
import grada.interfaces.InterfaceFunktionsListenAenderung;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.util.ArrayList;
import java.util.Iterator;

import javax.swing.JPanel;

/** FunktionenAuswahl. 
 * Diese Klasse bietet den Rahmen fr die FunktionePanels, in denen die 
 * Eigenschaften der Funktionen verndert werden knnen. 
 * @author Thilo Beckmann
 * @version 08.07.2005
 */
public class FunktionenAuswahl extends JPanel implements InterfaceFunktionsAenderung{
	private Insets abstand = new Insets(0,0,0,0);
	private ArrayList fuListe;
	private InterfaceFunktionsListenAenderung listener;
	
	public FunktionenAuswahl(ArrayList fuListe, InterfaceFunktionsListenAenderung listener){
		this.fuListe = fuListe;
		this.listener = listener;
		setLayout(new GridBagLayout());
		Funktion funktion;
		int i=0;
		if (fuListe.size()>0){
			Iterator iterator = fuListe.iterator();
			while (iterator.hasNext()){
				funktion = (Funktion)iterator.next();
				GridBagHinzufueger.add(this,new FunktionsPanel(funktion,this), 0,i++,	1,1,	1,1,	GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
			}
		}
		validate();
	}
	/** setzt die FunktionsListe.
	 * (wird zum aktualisieren der FunktionsListe bentigt)
	 * @param fuListe	neue FunktionsListe 
	 */
	public void setzeFunktionsListe(ArrayList fuListe){
		this.fuListe = fuListe;
		aktualisiere();
		validate();
	}
	/** aktualisiere alle FunktionsPanels.
	 * Hier wird verglichen, ob eine Funktion aus der Liste gelscht wurde.
	 * Falls ja wird ihr FunktionsPanel entfernt. Bei allen anderen Funktions-
	 * Panels wird die Farbe etc aktualisiert.
	 */
	private void aktualisiere(){
			int k=0;
			Funktion funktion;
			//loesche alle alten JPanel fr die Funktionen:
			Object elemente[] = getComponents();
			for (int i=0; i<elemente.length; i++){
				if (elemente[i] instanceof FunktionsPanel){
					if (((Component)elemente[i]).getName().equals(FunktionsPanel.ICHBINEINFUNKTIONSPANEL)){
						GridBagHinzufueger.remove(this,(Component)elemente[i]);
					}
				}
			}
			// und mache neue hin:
			if (fuListe.size()>0){
				Iterator iterator = fuListe.iterator();
				while (iterator.hasNext()){
					funktion = (Funktion)iterator.next();			
					GridBagHinzufueger.add(this,new FunktionsPanel(funktion,this), 0,k++,1,1,1,1,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
				}
			}

	}
	/** Wurde die Funktion verndert?
	 * wird aufgerufen, wenn in einem FunktionsPanel Eigenschaften der Funktion
	 * verndert wurden 
	 */
	public void funktionWurdeVeraendert(FunktionsEvent e){
		fuListe.remove(e.holeAlteFunktion());
		fuListe.add(fuListe.size(),e.holeFunktion());
		listener.funktionsListeWurdeVeraendert(new FunktionsListenEvent(fuListe));
	}
	/** Wurde die Funktion gelscht?
	 * wird aufgerufen, wenn in einem FunktionsPanel die Funktion mittels 
	 * "Lschen"-Button gelscht wurde.
	 */
	public void funktionsWurdeGeloescht(FunktionsEvent e){
		fuListe.remove(e.holeAlteFunktion());
		listener.funktionsListeWurdeVeraendert(new FunktionsListenEvent(fuListe));
	}
}
