package grada.steuereinheit;
import grada.event.FunktionsEvent;
import grada.geometriefiguren.Funktion;
import grada.interfaces.InterfaceFunktionsAenderung;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.util.ListResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JColorChooser;
import javax.swing.JDialog;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import sprachen.Sprachen;
/**
 * @author Thilo Beckmann
 * @version 03.07.05
 *
 * Diese Klasse ist dazu dar einfache nderungen an Funktionen vorzunehmen.
 */
public class FunktionsPanel extends JPanel implements ActionListener, FocusListener, ItemListener {
	private Funktion	funktion;
	private Funktion	alteFunktion;
	private JTextField	termFeld		= new JTextField();
	private JLabel		funktionsFarbe	= new JLabel(" ");
	private JButton		loeschen		= new JButton(new ImageIcon("icons/loeschen.png"));
	private JButton		uebernehmen		= new JButton(new ImageIcon("icons/uebernehmen.png"));
	private JButton		farbeAendern	= new JButton(new ImageIcon("icons/farbe.png"));
	private JCheckBox	anzeigen		= new JCheckBox("",true);
	private Insets		abstand			= new Insets(0,0,0,0);
	private Insets		panelAbstand	= new Insets(2,0,2,0);
	private InterfaceFunktionsAenderung listener	= null;
	private FunktionsEvent				fuEvent		= null;
	private Color ausgewaehlteFarbe;
	
	private Sprachen	sprachen = Sprachen.holeSprache();
	private ListResourceBundle	sp = sprachen.holeSprachBundle();
	
	/** FunktionsEvent.getFunktionsCommand gibt diesen String zurck, 
	 * wenn auf den Button "nderungen uebernehmen" geklickt wurde */
	public static final String	UEBERNEHMEAENDERUNG = "uebernehmeAenderung";
	/** FunktionsEvent.getFunktionsCommand gibt diesen String zurck, 
	 * wenn auf den Button "lsche Funktion" geklickt wurde */
	public static final String	LOESCHEFUNKTION		= "loescheFunktion";
	/** FunktionsEvent.getFunktionsCommand gibt diesen String zurck, 
	 * wenn auf den Button "Farbe ndern" geklickt wurde */
	public static final String	AENDERFARBE			= "aenderFarbe";
	/** Diese Konstante kann bei der Identifizierung eines FunktionsPanels helfen. */ 
	public static final String ICHBINEINFUNKTIONSPANEL = "ich bin ein FunktionsPanel";
	
	
	/** FunktionsPanel Konstruktor
	 * Mit diesem Konstruktor kann ein Panel mit Button, Checkboxen und Texteingabefeld erstellt werden.
	 * Dabei muss lediglich eine Funktion angegeben werden, aus der die ntigen Informationen zu
	 * den anzulegenden Einstellobjekten herausgeholt werden. 
	 * 
	 * @param funktion	Funktion, die mit einem Panel eingestellt werden soll
	 * @param listener	der Ereignislauscher (InterfaceFunktionsUebergabe) 
	 */
	public FunktionsPanel(Funktion funktion, InterfaceFunktionsAenderung listener){
		this.alteFunktion	= funktion;
		this.funktion		= funktion;
		this.listener		= listener;
		
		setLayout(new GridBagLayout());
		setName(ICHBINEINFUNKTIONSPANEL);
		GridBagHinzufueger.add(this,anzeigen,		0,1,	1,1,  0,0, GridBagConstraints.NONE,	GridBagConstraints.CENTER, abstand);
		GridBagHinzufueger.add(this,termFeld,		1,1,	1,1,999,0, GridBagConstraints.HORIZONTAL,	GridBagConstraints.CENTER, abstand);
		GridBagHinzufueger.add(this,uebernehmen,	2,1,	1,1,  0,0, GridBagConstraints.NONE,	GridBagConstraints.CENTER, abstand);
		GridBagHinzufueger.add(this,loeschen,		3,1,	1,1,  0,0, GridBagConstraints.NONE,	GridBagConstraints.CENTER, abstand);
		GridBagHinzufueger.add(this,farbeAendern,	5,1,	1,1,  0,0, GridBagConstraints.NONE,	GridBagConstraints.CENTER, abstand);
		
		termFeld.setText(funktion.holeFunktionsTerm());
		uebernehmen	.setSize(16,16);
		loeschen	.setSize(16,16);
		farbeAendern.setSize(30,16);
		
		anzeigen	.setToolTipText(sp.getString("FuPan_Diese Funktion anzeigen"));
		anzeigen	.setSelected(funktion.istSichtbar());
		setzeButtonEigenschaften (loeschen,
				new ImageIcon("grafik/icons/loeschen_normal.png"),
				new ImageIcon("grafik/icons/loeschen_rollover.png"),
				new ImageIcon("grafik/icons/loeschen_pressed.png"),
				false,false,false,sp.getString("FuPan_Diese Funktion loeschen"));
		setzeButtonEigenschaften (uebernehmen,
				new ImageIcon("grafik/icons/uebernehmen_normal.png"),
				new ImageIcon("grafik/icons/uebernehmen_rollover.png"),
				new ImageIcon("grafik/icons/uebernehmen_pressed.png"),
				false,false,false,sp.getString("FuPan_nderung bernehmen"));
		setzeButtonEigenschaften (farbeAendern,
				new ImageIcon("grafik/icons/farbeAendern_normal.png"),
				new ImageIcon("grafik/icons/farbeAendern_rollover.png"),
				new ImageIcon("grafik/icons/farbeAendern_pressed.png"),
				false,false,false,sp.getString("FuPan_Farbe dieser Funktion ndern"));
		anzeigen	.setMargin(abstand);
		funktionsFarbe	.setBackground(funktion.holeFarbe());
						 setBackground(funktion.holeFarbe());
		anzeigen		.setBackground(funktion.holeFarbe());
		uebernehmen		.setBackground(funktion.holeFarbe());
		loeschen		.setBackground(funktion.holeFarbe());
		farbeAendern	.setBackground(funktion.holeFarbe());
		funktionsFarbe.setOpaque(true);
		
		anzeigen	.addItemListener(this);
		termFeld	.addFocusListener(this);		
		uebernehmen	.addActionListener(this);
		loeschen	.addActionListener(this);
		farbeAendern.addActionListener(this);
		termFeld	.addActionListener(this);
		termFeld	.setActionCommand(UEBERNEHMEAENDERUNG);
		uebernehmen	.setActionCommand(UEBERNEHMEAENDERUNG);
		loeschen	.setActionCommand(LOESCHEFUNKTION);
		farbeAendern.setActionCommand(AENDERFARBE);
		termFeld.setBackground(funktion.holeFarbe());
		termFeld.setForeground(holeFordergrundFarbe(funktion.holeFarbe()));
	}
	/** setzt passend zu Hintergrundfarbe eine lesbare Fordergroundfarbe.
	 * 
	 * @param hintergrundFarbe	Hintergrundfarbe
	 * @return	gibt passende Fordergrundfarbe
	 */
	private Color holeFordergrundFarbe(Color hintergrundFarbe) {
		int r = hintergrundFarbe.getRed();
		int g = hintergrundFarbe.getGreen();
		int b = hintergrundFarbe.getBlue();
		if(r > 240 || g > 240) {
			return Color.black;
		} 
		else {
			return Color.white;
		}
	}
	/** setzt Eigenschaften von einem Button
	 * 
	 * @param but				JButton, der verndert wird
	 * @param normal			Image (normalzustand)
	 * @param rollover			Image (mouseover)
	 * @param pressed			Image (gedrckt)
	 * @param focuspaint		mit gemaltem focus?
	 * @param contentareapaint	mit hintergrund?
	 * @param borderpainted		mit rand?
	 * @param tooltiptext		ToolTipText
	 */
	private void setzeButtonEigenschaften(JButton but, ImageIcon normal, ImageIcon rollover, ImageIcon pressed, boolean focuspaint, boolean contentareapaint, boolean borderpainted, String tooltiptext){
		but	.setIcon		(normal);
		but	.setRolloverIcon(rollover);
		but	.setPressedIcon (pressed);
		but	.setFocusPainted(focuspaint);
		but	.setContentAreaFilled(contentareapaint);
		but	.setBorderPainted(borderpainted);
		but .setToolTipText(tooltiptext);
		but	.setMargin(abstand);
	}
	/** Setzt die Funktion des FunktionsPanels. 
	 * @param funktion	Funktion, die mit einem Panel eingestellt werden soll
	 */
	public void setzeFunktion(Funktion funktion){
		this.funktion = funktion;
		termFeld.setText(funktion.holeFunktionsTerm());
	}
	private void uebernehmeAenderungen(Funktion funktion, Funktion alteFunktion){
		listener.funktionWurdeVeraendert(new FunktionsEvent(funktion,alteFunktion));
	}
	
	
	public void actionPerformed(ActionEvent e){
		String cmd = e.getActionCommand();
		if (cmd==UEBERNEHMEAENDERUNG){
			funktion.setzeFunktionsTerm(termFeld.getText());
			listener.funktionWurdeVeraendert(new FunktionsEvent(funktion,alteFunktion));
		}
		else if (cmd==LOESCHEFUNKTION){
			listener.funktionsWurdeGeloescht(new FunktionsEvent(funktion,alteFunktion));
		}
		else if (cmd==AENDERFARBE){
			final JColorChooser farbauswaehler = new JColorChooser();
			ausgewaehlteFarbe = funktion.holeFarbe();
			ActionListener okListener = new ActionListener() {
				public void actionPerformed(ActionEvent e) {
					if (e.getActionCommand().equals("OK")){
						ausgewaehlteFarbe = farbauswaehler.getColor();
					}
					System.out.println("auswahl:  "+e.getActionCommand());
				}
			};
			farbauswaehler.setColor(funktion.holeFarbe());
			JDialog dialog = JColorChooser.createDialog(this,sp.getString("FuPan_Farbauswahl"),true,farbauswaehler,okListener,null);
			dialog.setVisible(true);
			
			funktion.setzeFarbe(ausgewaehlteFarbe);
			listener.funktionWurdeVeraendert(new FunktionsEvent(funktion,alteFunktion));
			
			funktionsFarbe	.setBackground(funktion.holeFarbe());
							 setBackground(funktion.holeFarbe());
			anzeigen		.setBackground(funktion.holeFarbe());
			uebernehmen		.setBackground(funktion.holeFarbe());
			loeschen		.setBackground(funktion.holeFarbe());
			farbeAendern	.setBackground(funktion.holeFarbe());
			termFeld		.setBackground(funktion.holeFarbe());
			termFeld		.setForeground(holeFordergrundFarbe(funktion.holeFarbe()));
		}
	}
	
	public void focusLost(FocusEvent e){
		System.out.println("FocusLost");
		if (!termFeld.getText().equals(funktion.holeFunktionsTerm())){
			funktion.setzeFunktionsTerm(termFeld.getText());
			listener.funktionWurdeVeraendert(new FunktionsEvent(funktion,alteFunktion));
		}
	}
	public void focusGained(FocusEvent e){}
	public void itemStateChanged(ItemEvent e){
		System.out.println("ItemChanged");
		funktion.setzeSichtbarkeit(anzeigen.isSelected());
		termFeld.setEditable(anzeigen.isSelected());
		listener.funktionWurdeVeraendert(new FunktionsEvent(funktion,alteFunktion));
	}
}
