package grada.steuereinheit;

import grada.event.ZoomEvent;
import grada.interfaces.InterfaceZoomAenderung;

import java.awt.Color;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.ListResourceBundle;

import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import sprachen.Sprachen;
/**
 * Diese Klasse stellt die ZoomBedienung zur Verfgung.
 * @author Thilo Beckmann
 */
public class ZoomBedienung extends JPanel implements ActionListener{
	private static final Color titelBackground	= new Color (100,139,203);
	private static final Color titelForeground	= new Color (255,255,255);
	private Sprachen	sprachen = Sprachen.holeSprache();
	private ListResourceBundle	sp = sprachen.holeSprachBundle();
	private static final Insets abstand	= new Insets (0,0,0,0);
	
	private int xzoom;
	private int yzoom;
	
	private InterfaceZoomAenderung listener;
	/** Setzt Attribute eines Labels.
	 * Diese Methode setzt die Attribute eines Labels, um dieses als berschrift 
	 * zur Gliederung der Oberflche hervorzuheben.  
	 * @param obj	JLabel, das als berschrift benutzt werden soll.
	 */
	private void setzeTitelEigenschaften (JLabel obj){
		obj.setBackground(titelBackground);
		obj.setForeground(titelForeground);
		obj.setOpaque(true);
		obj.setHorizontalAlignment(SwingConstants.CENTER);
		obj.setVisible(true);
	}
	/** setzt Eigenschaften von einem Button
	 * @param but				JButton, der verndert wird
	 * @param normal			Image (normalzustand)
	 * @param rollover			Image (mouseover)
	 * @param pressed			Image (gedrckt)
	 * @param focuspaint		mit gemaltem focus?
	 * @param contentareapaint	mit hintergrund?
	 * @param borderpainted		mit rand?
	 * @param tooltiptext		ToolTipText
	 */
	private void setzeButtonEigenschaften(
			JButton but, 
			ImageIcon normal, 
			ImageIcon rollover, 
			ImageIcon pressed, 
			boolean focuspaint, 
			boolean contentareapaint, 
			boolean borderpainted, 
			String tooltiptext){
		but	.setIcon		(normal);
		but	.setRolloverIcon(rollover);
		but	.setPressedIcon (pressed);
		but	.setFocusPainted(focuspaint);
		but	.setContentAreaFilled(contentareapaint);
		but	.setBorderPainted(borderpainted);
		but .setToolTipText(tooltiptext);
		but	.setMargin(abstand);
	}

	/** Erstellt die Zoom-Bedienelemente.
	 * Erstellt die Zoom-Bedienelemente (Button, zum Zoomen des Graphen)
	 */
	public ZoomBedienung(int xzoom, int yzoom, InterfaceZoomAenderung listener){
		this.xzoom = xzoom;
		this.yzoom = yzoom;
		this.listener = listener;
		JLabel		zoomTitel	= new JLabel(sp.getString("FuAnz_Zoom:"));
		JLabel		zoomLabel	= new JLabel(" ");
		ImageIcon	zoomin		= new ImageIcon();
		ImageIcon	zoomout		= new ImageIcon();
		ImageIcon	zoomxy		= new ImageIcon();
		ImageIcon	zoomxin		= new ImageIcon();
		ImageIcon	zoomxout	= new ImageIcon();
		ImageIcon	zoomyin		= new ImageIcon();
		ImageIcon	zoomyout	= new ImageIcon();
		JButton		bzoomin		= new JButton(zoomin);
		JButton		bzoomout	= new JButton(zoomout);
		JButton		bzoomxy		= new JButton(zoomxy);
		JButton		bzoomxin	= new JButton(zoomxin);
		JButton		bzoomxout	= new JButton(zoomxout);
		JButton		bzoomyin	= new JButton(zoomyin);
		JButton		bzoomyout	= new JButton(zoomyout);
		Insets		buttonicon	= new Insets(0,0,0,0);
		
		setLayout(new GridBagLayout());
		setzeTitelEigenschaften(zoomTitel);
		GridBagHinzufueger.add(this,zoomTitel,		0,0,	6,1,1,0,GridBagConstraints.HORIZONTAL,GridBagConstraints.CENTER,abstand);
		GridBagHinzufueger.add(this,zoomLabel,		4,1,	2,1,1,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomin,		0,1,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomout,		0,2,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomxy,		1,2,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomxin,		2,1,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomxout,		2,2,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomyin,		3,1,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		GridBagHinzufueger.add(this,bzoomyout,		3,2,	1,1,0,0,GridBagConstraints.NONE,GridBagConstraints.WEST,abstand);
		
		setzeButtonEigenschaften(bzoomout,
				new ImageIcon("grafik/icons/zoom-_normal.png"),
				new ImageIcon("grafik/icons/zoom-_rollover.png"),
				new ImageIcon("grafik/icons/zoom-_pressed.png"),
				false,false,false,sp.getString("FuAnz_Zoom verkleinern"));
		setzeButtonEigenschaften(bzoomin,
				new ImageIcon("grafik/icons/zoom+_normal.png"),
				new ImageIcon("grafik/icons/zoom+_rollover.png"),
				new ImageIcon("grafik/icons/zoom+_pressed.png"),
				false,false,false,sp.getString("FuAnz_Zoom vergrern"));
		setzeButtonEigenschaften(bzoomxy,
				new ImageIcon("grafik/icons/zoomxy_normal.png"),
				new ImageIcon("grafik/icons/zoomxy_rollover.png"),
				new ImageIcon("grafik/icons/zoomxy_pressed.png"),
				false,false,false,sp.getString("FuAnz_Zoom auf beiden Achsen gleich einstellen"));
		setzeButtonEigenschaften(bzoomxout,
				new ImageIcon("grafik/icons/zoom-x_normal.png"),
				new ImageIcon("grafik/icons/zoom-x_rollover.png"),
				new ImageIcon("grafik/icons/zoom-x_pressed.png"),
				false,false,false,sp.getString("FuAnz_X-Achse stauchen"));
		setzeButtonEigenschaften(bzoomxin,
				new ImageIcon("grafik/icons/zoom+x_normal.png"),
				new ImageIcon("grafik/icons/zoom+x_rollover.png"),
				new ImageIcon("grafik/icons/zoom+x_pressed.png"),
				false,false,false,sp.getString("FuAnz_X-Achse strecken"));
		setzeButtonEigenschaften(bzoomyout,
				new ImageIcon("grafik/icons/zoom-y_normal.png"),
				new ImageIcon("grafik/icons/zoom-y_rollover.png"),
				new ImageIcon("grafik/icons/zoom-y_pressed.png"),
				false,false,false,sp.getString("FuAnz_Y-Achse stauchen"));
		setzeButtonEigenschaften(bzoomyin,
				new ImageIcon("grafik/icons/zoom+y_normal.png"),
				new ImageIcon("grafik/icons/zoom+y_rollover.png"),
				new ImageIcon("grafik/icons/zoom+y_pressed.png"),
				false,false,false,sp.getString("FuAnz_Y-Achse strecken"));
		
		bzoomin		.addActionListener(this);
		bzoomout	.addActionListener(this);
		bzoomxy		.addActionListener(this);
		bzoomxin	.addActionListener(this);
		bzoomxout	.addActionListener(this);
		bzoomyin	.addActionListener(this);
		bzoomyout	.addActionListener(this);
		bzoomin		.setActionCommand("zoom+");
		bzoomout	.setActionCommand("zoom-");
		bzoomxy		.setActionCommand("zoomxy");
		bzoomxin	.setActionCommand("zoom+x");
		bzoomxout	.setActionCommand("zoom-x");
		bzoomyin	.setActionCommand("zoom+y");
		bzoomyout	.setActionCommand("zoom-y");
	}
	public void actionPerformed(ActionEvent e){
		String cmd = e.getActionCommand();
		if 		(cmd == "zoom+"){	xzoom--;	yzoom--;}
		else if (cmd == "zoom-"){	xzoom++;	yzoom++;}
		else if (cmd == "zoom+x"){	xzoom--;}
		else if (cmd == "zoom-x"){	xzoom++;}
		else if (cmd == "zoom+y"){	yzoom--;}
		else if (cmd == "zoom-y"){	yzoom++;}
		else if (cmd == "zoomxy"){	yzoom=xzoom;}
		listener.zoomWurdeVeraendert(new ZoomEvent(xzoom,yzoom));
		
	}
}
