/*
 * Ein Paket, das alle Klassen enthlt, die ntig zum Parsen von Funktionstermen sind.
 */
package parser;

import grada.interfaces.InterfaceParserEingabe;
import grada.geometriefiguren.Punkt2d;


import java.util.ArrayList;
import java.util.HashMap;
import java.util.ListResourceBundle;

import parser.baeume.Baum;
import parser.baeume.BaumErzeuger;
import parser.baeume.Polynompruefer;
import sprachen.Sprachen;

/**Eine Klasse, die zustndig ist fr das erstellen von Funktionen aus Strings.
 * Sie berechnet nach dem Anlegen einer Funktion die Werte, die man anfordert und gibt diese
 * als Array zurck.<br>
 * Ist ein Funktionsterm fehlerhaft, wird ein Fenster geffnet, das zur Korrektur auffordert.
 * Der Parser ist zwar kein Singleton, aber kann seine eigene Instanz mit Hilfe einer statischen Methode zurckgeben. 
 * @author Stephan Hagebck
 * Datum: 11.09.2005
 */
public class Parser {
	/**Verhindert das Aktualisieren der Registerkarten, wenn dadurch eine falsche Funktion immer wieder Fehler hervorrufen wrde.*/
    public static boolean aktualisierenErlaubt=true;
    
    /**Beeinflusst die Gre des Funktionenspeichers. Werden mehr als 75 eingegeben, wird der Speicher erweitert.*/
	private final int anz_funktionen=100;
	
	/**Speichert alle eingegebenen Funktionsterme*/
	private HashMap speicher;
	/**Ein Frame zur Korrektur fehlerhafter Funktionsterme*/
	private Fehlermeldung fehlermeldung;
	/**Diese Klasse ist der eigentliche Parser und stellt die Bume her.*/
	private BaumErzeuger baumErzeuger;
	
	/**Enhlt smtliche Werte der Parameter, die eingegeben werden knnen. [0] entspricht a, [25] z.*/
	static public double[] parameter;
	/**Die Instanz des Parsers. Enhlt eine Referenz, sobald der Konstruktor aufgerufen wird.*/
	private static Parser instanz;
	/**Referenz auf das aufrufende Bedienfeld, um dort einen korrigierten Term abzuliefern.*/
	private InterfaceParserEingabe bedienfeld;
	/**Gibt an, wie nah die Ableitungswerte bei iterativer Bestimmung neben dem x-Wert liegen. 1000 entspricht 1/1000 Intervallbreite.*/
	private final int GENAUIG_ABL=1000;
	
	 /**Zustndig fr das holen der sprachspezifischen Strings.*/
    private ListResourceBundle	sp = Sprachen.holeSprache().holeSprachBundle();
	
	
	/** Ein neuer Parser wird erzeugt.<br><br><b><u>Hinweis:</u></b> Der Parser sollte nur einmal instanziert werden, weil es sinnvoller ist,
	 * einen Parser fr alle Funktionen zu nehmen (Speicherverbrauch). Mit <b>static getInstance()</b> bekommt man eine Instanz des Parsers, die auch schon
	 * anderswo im Programm benutzt werden knnte. Wer einen neuen und leeren Parser mchte, der benutzt den Konstruktor.
	 */
	public Parser() {
		fehlermeldung=new Fehlermeldung();
		speicher=new HashMap(anz_funktionen, (float) 0.75);
		baumErzeuger=new BaumErzeuger();
		//Fr den Fall, dass jemand einen neuen leeren Parser mchte (warum auch immer), soll der Standardparser nicht verloren gehen
		if (instanz==null) instanz=this;
	}
	
	
	/**Diese Methode liefert Funktionswerte in Abhngigkeit von x.
	 * Man liefert ein Intervall, in dem die Werte berechnet werden sollen.
	 * Nur aufsteigende Intervalle sind zulssig.
	 * @param Funktionsterm Der String mit dem Funktionsterm, dessen Werte man haben mchte
	 * @param Anfang Beginn des Intervalls
	 * @param Intervall Schrittgre
	 * @param Ende Ende des Intervalls (Ende eingeschlossen, wenn Anfang + n*Intervall =Ende)
	 * @param Bedienfeld Die Instanz, die das Eingabefeld fr den Term sowie die Scrollbalken fr die Parameter enthlt.
	 * @return ArrayList Liste mit Punkten der Funktion, nach aufsteigender x-Koordinate sortiert.
	 * @throws IllegalArgumentException bei falschen Parametern
	 */
	public ArrayList gibWerte(String Funktionsterm, double Anfang, double Intervall, double Ende, InterfaceParserEingabe Bedienfeld) throws IllegalArgumentException{
		Funktionsterm=Funktionsterm.trim();
		//Funktion nicht vorhanden, also wird sie angelegt. Will der Nutzer eine falsche Funktion nicht berichtigen, werden keine Werte weitergegeben.
	    if (!speicher.containsKey(Funktionsterm)) {
	        this.bedienfeld=Bedienfeld;
	        if (!neueFunktion(Funktionsterm)) return new ArrayList();
	    }
		
		if (Anfang>Ende) 			throw new IllegalArgumentException("Unzulssiger Wertebereich: Anfang="+Anfang+" Ende="+Ende);
		if (Anfang+Intervall>Ende)  throw new IllegalArgumentException("Intervall bersteigt Endwert: "+Intervall);
		if (Intervall<0) 			throw new IllegalArgumentException("Nur positive Intervalle sind zulssig.");
		if (Bedienfeld==null) 		throw new IllegalArgumentException("InterfaceParserEingabe ist \"null\"");
		
		Baum funktion;
		//Aus dem Speicher wird der Funktionsbaum geholt
		funktion=((Funktionenspeicher)speicher.get(Funktionsterm)).Funktionsbaum;
		parameter=Bedienfeld.holeTermParameter();
		double[] Funktionswerte=funktion.gibWerte(Anfang, Intervall, Ende);
		
		ArrayList rueckgabe =new ArrayList(Funktionswerte.length);
		for (int i=0; i<Funktionswerte.length; i++) {
		    	//						  x-Koordinate		  y-Koordinate
				rueckgabe.add(new Punkt2d(Anfang+i*Intervall, Funktionswerte[i]));
		}
		
		return rueckgabe;
	}
	
	
	/**Diese Methode liefert Funktionswerte der Ableitung in Abhngigkeit von x.
	 * Man liefert ein Intervall, in dem die Werte berechnet werden sollen.
	 * Nur aufsteigende Intervalle sind zulssig.
	 * Diese Methode arbeitet algebraisch, sie bestimmt die Steigung der Funktion.
	 * @param Funktionsterm Der String mit dem Funktionsterm, dessen Werte man haben mchte
	 * @param Anfang Beginn des Intervalls
	 * @param Intervall Schrittgre
	 * @param Ende Ende des Intervalls (Ende eingeschlossen, wenn Anfang + n*Intervall =Ende)
	 * @param Bedienfeld Die Instanz, die das Eingabefeld fr den Term sowie die Scrollbalken fr die Parameter enthlt.
	 * @param Grad Der Grad der Ableitung: 0 ist die Funktion selber, 1 die erste...
	 * @return ArrayList Liste mit Punkten der Funktion, nach aufsteigender x-Koordinate
	 * @throws IllegalArgumentException bei falschen Parametern, insbesondere, wenn der Funktionsterm nicht stimmt
	 */
	public ArrayList gibWerteAbgeleitet(String Funktionsterm, double Anfang, double Intervall, double Ende, InterfaceParserEingabe Bedienfeld, int Grad) throws IllegalArgumentException{
		Funktionsterm=Funktionsterm.trim();
		//Funktion nicht vorhanden, also wird sie angelegt. Will der Nutzer eine falsche Funktion nicht berichtigen, werden keine Werte weitergegeben.
	    if (!speicher.containsKey(Funktionsterm)) {
	        this.bedienfeld=Bedienfeld;
	        if (!neueFunktion(Funktionsterm)) return new ArrayList();
	    }
		
		if (Anfang>Ende) 			throw new IllegalArgumentException("Unzulssiger Wertebereich: Anfang="+Anfang+" Ende="+Ende);
		if (Anfang+Intervall>Ende)  throw new IllegalArgumentException("Intervall bersteigt Endwert: "+Intervall);
		if (Intervall<0) 			throw new IllegalArgumentException("Nur positive Intervalle sind zulssig.");
		if (Bedienfeld==null) 		throw new IllegalArgumentException("InterfaceParserEingabe ist \"null\"");
		if (Grad<1)					throw new IllegalArgumentException("Nur Grade der Ableitung >0 sind zulssig.");
		
		
		Funktionenspeicher fSpeicher=(Funktionenspeicher)speicher.get(Funktionsterm);
		Baum funktion = fSpeicher.Funktionsbaum;
		ArrayList ableitungen=fSpeicher.Ableitungen;
		
		//Es wird so oft wie erforderlich abgeleitet und die Ableitungen im Funktionenspeicher gespeichert.
		for (int i=ableitungen.size(); i<=Grad; i++) {
		    if (i==0) ableitungen.add(funktion);
		    else {
		        //Aus der Liste der Ableitungen wird die Funktion nchsthheren Grades genommen und abgeleitet, dann gespeichert.
		        Baum b=((Baum)ableitungen.get(i-1)).leiteAb();
		        System.out.println(i+". Ableitung                   :"+b.toString());
		        String alterBaum;
		        do {
		            alterBaum=b.toString();
		            b=b.vereinfache();
		            //Ich vergleiche die Strings, weil equals fr alle Bume zu schreiben sehr aufwndig wre. ==0 bedeutet Identitt
		        } while(alterBaum.compareTo(b.toString())!=0);
		        ableitungen.add(b);
		        System.out.println(i+". Ableitung nach Vereinfachung:"+ableitungen.get(i).toString());
		    }
		}
		
		//analog zu gibWerte()
		funktion = (Baum)ableitungen.get(Grad);
		parameter=Bedienfeld.holeTermParameter();
		double[] Funktionswerte=funktion.gibWerte(Anfang, Intervall, Ende);
		
		ArrayList rueckgabe =new ArrayList(Funktionswerte.length);
		for (int i=0; i<Funktionswerte.length; i++) {
			rueckgabe.add(new Punkt2d(Anfang+i*Intervall, Funktionswerte[i]));
		}
		
		return rueckgabe;
		
		
		//Mglichkeit, Ableitungen durch Bestimmung der Steigung zu bestimmen.
		/*if (Grad==1) { 
		    Baum funktion;
		    funktion=((Funktionenspeicher)speicher.get(Funktionsterm)).Funktionsbaum;
		    parameter=Bedienung.holeTermParameter();
		    
		    double dx=Intervall/GENAUIG_ABL;
		    ArrayList funktionswerteVor =gibWerte(Funktionsterm, Anfang-dx, Intervall, Ende-dx, Bedienung);
		    ArrayList funktionswerteNach=gibWerte(Funktionsterm, Anfang+dx, Intervall, Ende+dx, Bedienung);
		    ArrayList rueckgabe=new ArrayList(funktionswerteVor.size());
		    double y;
		    for (int i=0; i<funktionswerteVor.size(); i++) {
		        //				y2							   -				y1							=dy
		        y=(((Punkt2d)funktionswerteNach.get(i)).holeY()-((Punkt2d)funktionswerteVor.get(i)).holeY())     /dx;
		        rueckgabe.add(new Punkt2d(Anfang+i*Intervall, y));
		    }
		    return rueckgabe;
		}
		//else {
		    double dx=Intervall/GENAUIG_ABL;
		    ArrayList funktionswerteVor =gibWerteAbgeleitet(Funktionsterm, Anfang-dx, Intervall, Ende-dx, Bedienung, Grad-1);
		    ArrayList funktionswerteNach=gibWerteAbgeleitet(Funktionsterm, Anfang+dx, Intervall, Ende+dx, Bedienung, Grad-1);
		    ArrayList rueckgabe=new ArrayList(funktionswerteVor.size());
		    double y;
		    for (int i=0; i<funktionswerteVor.size(); i++) {
		        //				y2							   -				y1							=dy
		        y=(((Punkt2d)funktionswerteNach.get(i)).holeY()-((Punkt2d)funktionswerteVor.get(i)).holeY())     /dx;
		        rueckgabe.add(new Punkt2d(Anfang+i*Intervall, y));
		    }
		    return rueckgabe;
		//}*/
	}
	
	
	
	
	/**Diese Methode liefert Funktionswerte in Abhngigkeit von x.
	 * Man liefert ein Intervall, in dem die Werte berechnet werden sollen.
	 * Nur aufsteigende Intervalle sind zulssig
	 * 
	 * Die Werte werden hier im Baum berechnet, kleiner Performancevorteil.
	 * @param Funktionsterm Der String mit dem Funktionsterm, dessen Werte man haben mchte
	 * @param Anfang Beginn des Intervalls
	 * @param Intervall Schrittgre
	 * @param Ende Ende des Intervalls (Ende eingeschlossen, wenn Anfang + n*Intervall =Ende)
	 * @return ArrayList Liste mit Punkten der Funktion, nach aufsteigender x-Koordinate
	 * @throws IllegalArgumentException bei falschen Parametern
	 */
	protected double[] gibWerte(String Funktionsterm, double Anfang, double Intervall, double Ende) throws IllegalArgumentException{
		Funktionsterm=Funktionsterm.trim();
		//Funktion nicht vorhanden, also wird sie angelegt. Will der Nutzer eine falsche Funktion nicht berichtigen, werden keine Werte weitergegeben.
	    if (!speicher.containsKey(Funktionsterm)) {
	        if (!neueFunktion(Funktionsterm)) return null;
	    }
		
		if (Anfang>Ende) 			throw new IllegalArgumentException("Unzulssiger Wertebereich: Anfang="+Anfang+" Ende="+Ende);
		if (Anfang+Intervall>Ende)  throw new IllegalArgumentException("Intervall bersteigt Endwert: "+Intervall);
		if (Intervall<0) 			throw new IllegalArgumentException("Nur positive Intervalle sind zulssig.");
		
		Baum funktion;
		funktion=((Funktionenspeicher)speicher.get(Funktionsterm)).Funktionsbaum;
		double[] Funktionswerte=funktion.gibWerte(Anfang, Intervall, Ende);
		return Funktionswerte;
	}
	
	
	/**Eine neue Funktion wird angelegt.
	 * Ist der Funktionsterm fehlerhaft, bricht die Methode ab
	 * und der Nutzer wird zur Korrektur aufgefordert. Ist der Funktionsterm schon vorhanden,
	 * passiert nichts.
	 * @param Funktionsterm Ein String, der die Funktion enthlt.
	 * @return <b>true</b>, wenn die Funktion geparst werden konnte<br><b>false</b> fr eine falsche Funktion, die der Nutzer nicht korrigieren wollte.
	 */
	public boolean neueFunktion(String Funktionsterm) {
		if (!speicher.containsKey(Funktionsterm)) {
			Baum funktionsbaum;
			System.out.println("\nParser startet: "+Funktionsterm);
			
			
		    try {
		        Funktionsterm=Funktionsterm.trim();
		        if (Funktionsterm.equals("")) throw new Lesefehler(sp.getString("Parser_Der Funktionsterm ist leer!"), Funktionsterm);
		        //Das eigentliche Parsen
		        funktionsbaum=baumErzeuger.neuerBaum(Funktionsterm);
		        
				System.out.println("Funktion aus dem Parser mit toString(): "+funktionsbaum.toString());
				funktionsbaum=funktionsbaum.vereinfache();
				System.out.println("Funktion nach dem Vereinfachen        : "+funktionsbaum.toString()+"\n");
				//													String		   Baum			  ein HashSet. Es wird in eine Liste der Parameter umgewandelt.
				Funktionenspeicher funktion= new Funktionenspeicher(Funktionsterm, funktionsbaum, baumErzeuger.vorhandeneParameter);
				speicher.put(Funktionsterm, funktion);
				this.bedienfeld=null;
				return true;
		    }
		    catch (Lesefehler e) {
		        aktualisierenErlaubt=false;
		        fehlermeldung.neuerFehler(e.getMessage(), e.Funktionsterm(), e.getStackTrace().toString(), bedienfeld);
		        this.bedienfeld=null;
		        return false;
		    }
		    catch (Exception e) {
		        System.out.println("Nicht behandelte Exception aus dem Baum. Dies ist ein Programmfehler. Schicken Sie uns doch eine Email.");
		        e.printStackTrace();
		        this.bedienfeld=null;
		        return false;
		    }
		    
		}
        //Stellt den compiler zufrieden, kommt aber nicht vor, ein Schleifeneintritt bei true zwingend geschieht.
		return false;
	}

	
	
	
	/**Diese Methode liefert einen Funktionswert als double zurck.
	 * Wenn die Funktion existiert, wird sie aus dem Speicher geholt und der Wert berechnet.
	 * Existiert sie noch nicht, wird sie neu angelegt.
	 * @param x Der x-Wert, dessen y-Wert man haben mchte.
	 * @param Funktionsterm Der Funktionsterm, der den Wert berechnet.
	 * @return <b>f(x)</b>, den Funktionswert.
	 */
	public double f(double x, String Funktionsterm, InterfaceParserEingabe Bedienfeld) {
		Funktionsterm=Funktionsterm.trim();
		if (!speicher.containsKey(Funktionsterm)) {
		    this.bedienfeld=Bedienfeld;
		    if (!neueFunktion(Funktionsterm)) {
		        return 0.0;
		    }
		}
		Baum funktion=((Funktionenspeicher)speicher.get(Funktionsterm)).Funktionsbaum;
		
		return funktion.f(x);
	}
	
	
	
	/**Diese Methode liefert Funktionswerte in einer Hashmap gespeichert zurck.
	 * @param Funktionsterm
	 * @param A Anfang des Intervalls (x-Wert)
	 * @param I	Intervalllnge
	 * @param E Ende des Intervalls (Ende eingeschlossen)
	 * @return Eine HashMap mit den x-Werten als Schlssel und den dazugehrenden y-Werten
	 */
	public HashMap funktionswerte(String Funktionsterm, double A, double I, double E, InterfaceParserEingabe Bedienfeld) {
	    this.bedienfeld=Bedienfeld;
	    double[] yWerte=gibWerte(Funktionsterm, A, I, E);
	    this.bedienfeld=null;
	    if (yWerte==null) return new HashMap();
	    HashMap rueckgabe =new HashMap(yWerte.length);
	    for (int i=0; i<yWerte.length; i++) {
	        rueckgabe.put(new Double(A+i*I),new Double(yWerte[i]));
	    }
	    return rueckgabe;
	}
	
	
	
	
	
	/**Prft, ob ein Funktionsterm ein Polynom ist. Zugelassen ist nur eine Eingabe in der Form:
	 * <u>ax^3+bx^2+cx^+d</u>, wobei a,b,c,d nur natrliche Zahlen sind.
	 * @param Polynom Der Funktionsterm des vermeintlichen Polynoms
	 * @return <b>double[11]</b><br>Auf [5] der Faktor vor x^5. <br> <b>null</b>, wenn der Term kein Polynom ist. 
	 */
	public double[] parameterDesPolynoms(String Polynom, InterfaceParserEingabe Bedienfeld) {
	    Polynom=Polynom.trim();
		if (!speicher.containsKey(Polynom)) {
	        this.bedienfeld=Bedienfeld;
	        if (!neueFunktion(Polynom)) {
	            return null;
	        }
	    }
		Baum funktion=((Funktionenspeicher)speicher.get(Polynom)).Funktionsbaum;
		if (funktion==null) return null;
		if (getParameter(Polynom).size()!=0) return null;
		
		Polynompruefer p=new Polynompruefer();
		return p.pruefePolynom(funktion);
	}
	
	
	/**Gibt eine (geordnete) Liste mit den Parametern, die in der Funktion verwendet werden zurck.
	 * @param Funktion Der Funktionsterm
	 * @return Eine ArrayList mit den Parametern der Funktion (alphabetisch)
	 */
	public ArrayList getParameter(String Funktion) {
		Funktion=Funktion.trim();
		return ((Funktionenspeicher)speicher.get(Funktion)).Parameter;
	}
	
	/**Gibt die Instanz des Parsers zurck
	 *@return Die Instanz des Parsers, soweit sie existiert. Wenn nicht, wird eine instanziert. 
	 */
	public static Parser getInstance() {
	    if (instanz==null) new Parser();
	    return Parser.instanz;
	}
}
