package parser.baeume;

import java.util.ArrayList;



/**Abstrakte Klasse, von der alle Bume, die eine Funktion reprsentieren, erben. Dies stellt
 * gemeinsame Methoden sicher, damit sich alle Bume gleich benutzen lassen. Dank des Polymorphismus
 * muss so auch nicht feststehen, welche Art Baum gespeichert wird, weil alle Bume Erben dieser Klasse sind.
 * @author Stephan Hagebck
 */
public abstract class Baum implements Cloneable{
    /**Die in der Hierarchie unteren Bume.*/
    protected ArrayList liste;
    /**Wenn der gesamte Baum negativ werden soll, weil ein - voransteht*/
    protected boolean negativ;
    
    /**Gibt den Funktionswert in Abhngigkeit von x zurck.
     * @param x
     * @return <b>y</b>, der Funktionswert
     */
    abstract public double f(double x);
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    abstract public String toString();
    
    /**Leitet den Baum ab.
     * @return Der Baum der Ableitung.*/
    abstract public Baum leiteAb();
    
    /**Vereinfacht den Baum (z.B. Zusammenfassen von 5+7 zu 12).
     * @return Der vereinfachte Baum.
     */
    abstract public Baum vereinfache();
    
    
    /**Diese Methode macht den Rckgabewert des Baumes negativ.*/
    void negativ() {
        if (negativ) negativ=false; 
        else negativ = true;
    }
    
    /**Diese Methode hngt einen neuen Baum an den bestehenden an.
     * @param baum Der anzuhngende Baum 
     */
    void haengeAn(Baum baum) {
        liste.add(baum);
    }
    
    
    /**Diese Methode liefert Funktionswerte in Abhngigkeit von x.
	 * Man liefert ein Intervall, in dem die Werte berechnet werden sollen.
	 * 
	 * @param Anfang Beginn des Intervalls
	 * @param Intervall Schrittgre
	 * @param Ende Ende des Intervalls (Ende eingeschlossen)
	 * @return double[] Ein Array mit den y-Werten, nach aufsteigenden x-Werten geordnet
	 */
	public double[] gibWerte(double Anfang, double Intervall, double Ende) {
		int AnzWerte = (int)((Ende-Anfang)/Intervall)+2;
		double Werte[]=new double[AnzWerte];
		
		for (int i=0; i<AnzWerte; i++) {
			Werte[i]=f(Anfang+i*Intervall);
		}
		return Werte;
	}
	
	
	
	/**Kontolliert, ob in einem der unteren Bume ein x vorkommt. Ist dies nicht der Fall,
	 * braucht dieser Baum nicht abgeleitet zu werden.
	 * @return <b>true</b>, wenn x vorhanden, sonst <b>false</b>.
	 */
	protected boolean xVorhanden() {
	    Baum b;
	    boolean xGefunden=false;
	    
	    for (int i=0; i<liste.size() && !xGefunden; i++) {
	        b=(Baum)liste.get(i);
	        if (b.xVorhanden()) xGefunden=true;
	    }
	    return xGefunden;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
	    Baum rueckgabe=null;
	    try {
	        rueckgabe=(Baum)super.clone();
	        //Da nur primitive Datentypen geklont werden:
	        if (liste!=null) rueckgabe.liste=(ArrayList)liste.clone();
	    }
	    catch (CloneNotSupportedException e) {
	        //passiert aber nicht, denn Baum implementiert Cloneable
	    }
	    
	    
	    //Primitive Datentypen werden ja kopiert...
	    return rueckgabe;
	}
}
