package parser.baeume;

import parser.Lesefehler;
import java.text.NumberFormat;
import java.text.ParsePosition;
import java.util.HashSet;
import java.util.ListResourceBundle;

import sprachen.Sprachen;

/**Diese Klasse ist der eigentliche Parser und legt auf Basis eines Strings eine Baumstruktur an, 
 * die den Funktionsterm reprsentiert.
 * @author Stephan Hagebck
 */
public class BaumErzeuger {
    /**Enthlt die monadischen Operatoren, nach denen gesucht wird.
     * Ihr Index in diesem Array dient gleichzeitig zu ihrer Identifikation.*/
    protected static final String monOperatoren[] =new String[]{
			"sin",	//0 
			"cos",	//1 
			"tan",	//2 
			"arcsin",//3 
			"arccos",//4 
			"arctan",//5 
			"ln",	//6 
			"sqrt"	//7 
		};
    
    /**Erhlt von den Bumen smtliche Parameter als chars. Sie werden einfach hinzugefgt, sobald ein neuer Parameter auftaucht.*/
    public static HashSet vorhandeneParameter; 
    /**Zustndig fr das holen der sprachspezifischen Strings.*/
    private ListResourceBundle	sp = Sprachen.holeSprache().holeSprachBundle();
    
    /**Instanziert einen neuen Baumerzeuger, der die Funktionsterme in Bume bersetzt.
     * 
     */
    public BaumErzeuger(){
        
    }
    
    
    /**Parst den Funktionsterm und schreibt ihn in eine Baumstruktur.
     * @param Funktionsterm Der zu parsende Term
     * @return Den Baum, der um Werte der Funktion errechnen kann.
     * @throws <b>Lesefehler</b> Wenn ein Funktionsterm fehlerhaft ist, tritt diese Exception auf.
     */
    public Baum neuerBaum(String Funktionsterm) throws Lesefehler, Exception {
        vorhandeneParameter=new HashSet();
        try {
            Funktionsterm=Funktionsterm.replace('.', ',');
            return lese(Funktionsterm, 1, false);
        }
        catch (Lesefehler e) {
            e.setzeFunktionsterm(Funktionsterm);
            throw e;
        }
    }
    
    /**Parst den Funktionsterm.
     * @param F Der zu parsende Term
     * @param typ Verkrzt die bentigte Zeit, da eine Suche nach Rechenzeichen, die nicht mehr vorkommen, bersprungen wird.
     * @return Baum Verschiedene Arten von Bumen, die den Funktionsterm wiedergeben.
     * @throws <b>Lesefehler</b> Wenn ein Funktionsterm fehlerhaft ist, tritt diese Exception auf.
     */
    private Baum lese(String F, int typ, boolean negativ) throws Lesefehler, Exception {
        F=F.trim();
        if (F.equals("")) throw new Lesefehler(sp.getString("Parser_Rechenzeichen"), F);
        boolean innerhalbKlammer[] = new boolean[F.length()];
        int klammertiefe=0;
        int letzterOp=F.length();
        int klammerAuf=0, klammerZu=0;
        Baum baum=null;
        
        
        //Suche nach Klammern und Speicherung der Positionen (wenn keine Klammern vorhanden, ist innerhalbKlammer[0 --> length-1]==false)
        while (F.indexOf('(')!=-1 || F.indexOf(')')!=-1) {
            for (int i=0; i<F.length(); i++) {
                switch (F.charAt(i)) {
        			case '(':
        			    klammerAuf++;
        			    klammertiefe++;
        				innerhalbKlammer[i]=true;
        				break;
        			
        			case ')':
        				klammerZu++;
        				if (--klammertiefe < 0) throw new Lesefehler(sp.getString("Parser_Klammerpos"), F); 
        				//Kein break, da an der Position der letzten ) FALSE stehen soll, sonst aber TRUE.
        			
    				//da inneralbKlammer[i] standardmig false
        			default: if (klammertiefe>0) innerhalbKlammer[i]=true;
                }
            }
            if (klammerAuf != klammerZu) throw new Lesefehler(sp.getString("Parser_Klammerzahl"), F); 
        
            //Da nun die Position der Klammern bekannt ist, kann (2x+5) zu 2x+5 werden.
            boolean ganzerTermInKlammern=true;
            for (int i=0; i<innerhalbKlammer.length-1; i++) {
                if (!innerhalbKlammer[i]) {
                    ganzerTermInKlammern=false;
                    break;
                }
            }
            //Nun werden die Klammern vorne und hinten gelscht. Es muss noch einmal gesucht werden, da sich die Lnge des Terms ndert.
            if (ganzerTermInKlammern) {
                F=F.substring(1, F.length()-1);
                innerhalbKlammer = new boolean[F.length()];
                letzterOp=F.length();
                typ=1;
            }
            else break;
        }
        
        
        
        
        //Dieser switch dient dazu, eine Suche nach z.B. Punkt- und Strichrechnung zu berspringen, wenn diese schon durchgefhrt wurde.
        switch (typ) {
        
        //Strichrechnung
        case 1:
            //Wird bersprungen, wenn kein + oder -           	
            if (F.indexOf('+')!=-1 || F.indexOf('-')!=-1) {
                baum=new Strichrechnung();
                if (negativ) baum.negativ();
           	    for (int i=F.length()-1; i>-1; i--) {
                    switch (F.charAt(i)) {
                		//			 v == auerhalb, habe die booleans aber so gewhlt, weil nach Initialisierung immer FALSE --> Wenn keine Klammern da sind, braucht keiner der booleans berschrieben zu werden.
                    	case '+': if(!innerhalbKlammer[i] && i!=0 && (F.charAt(i-1)!='*' && F.charAt(i-1)!='/' && F.charAt(i-1)!='^')) {baum.haengeAn(lese(F.substring(i+1, letzterOp), 2, false)); letzterOp=i;} break;
                		//Die Bedingung: Wenn auerhalb einer Klammer, 
                    	case '-': if(!innerhalbKlammer[i] && i!=0 && (F.charAt(i-1)!='*' && F.charAt(i-1)!='/' && F.charAt(i-1)!='^')) {baum.haengeAn(lese(F.substring(i+1, letzterOp), 2, true)); letzterOp=i;} break;
                		default: break;
                    }
                }
                //Wird ein Operator gefunden, werden neue Bume erzeugt. Im Folgenden wird der erste Summand angehngt, da die obige Schleife schon frher abbricht.
                //Fr den Fall, dass kein Operator gefunden wurde (--> wenn s.u. false), weiter bei der Punktrechnung. Falls ein - an erster Stelle, wird auch kein Summand angehngt.
                if (letzterOp!=F.length()) {
                    if (F.charAt(0)!='-') baum.haengeAn(lese(F.substring(0, letzterOp), 2, false));
                    else baum.haengeAn(lese(F.substring(1, letzterOp), 2, true));
                    return baum;
        //Wenn s.o. == TRUE, ist die Methode hier beendet.
                }
           	}
                
         
        
        //Punktrechnung
        case 2:
            //Wenn keine Punktrechnung mehr stattfindet, zu case 3.
            if (F.indexOf('*')!=-1 || F.indexOf('/')!=-1) {
                Punktrechnung baumPkt=new Punktrechnung();
                if (negativ) baumPkt.negativ();
                for (int i=F.length()-1; i>-1; i--) {
                    switch (F.charAt(i)) {
                		case '*': if(!innerhalbKlammer[i]) {baumPkt.haengeAn(lese(F.substring(i+1, letzterOp), 3, false)); letzterOp=i;} break;
                		case '/': if(!innerhalbKlammer[i] && i!=0) {baumPkt.haengeAnDiv(lese(F.substring(i+1, letzterOp), 3, false)); letzterOp=i;} break;
                		default: break;
                    }
                }
                //Analog zur Strichrechnung
                if (letzterOp!=F.length()) {
                    baumPkt.haengeAn(lese(F.substring(0, letzterOp), 3, false));
                    return baumPkt;
        /*Wenn s.o. == TRUE, ist die Methode hier beendet.*/
                }
            }
            
        
        
        
        //Monadische Operatoren
        case 3:
            //naechster[0] ist der Typ des Operators, [1] seine Position, [2] die erste Position HINTER dem Operator
            int[] naechster = naechsterMonadischer(F, innerhalbKlammer, 0); 
            if (naechster[2]==F.length()) throw new Lesefehler(sp.getString("Parser_Op")+F.substring(naechster[1], naechster[2])+sp.getString("Parser_Argument"), F);  //$NON-NLS-2$
            
            //	v Kein Operator oder 					v z.B. x^sin25 --> weiter bei case4
            if (naechster[0]!=-1 && (naechster[1]==0 || F.charAt(naechster[1]-1)!='^')) {
            //						 ^ verhindert outOfRangeException von ^ wenn der Term z.B. "sinx" und damit naechster[1] == 0
                
                //Kontrolle auf richtige Syntax fr das Argument des Operators
                //Steht eine Klammer hinter dem Operator ...
                if (F.charAt(naechster[2])=='(') for (int i=naechster[2]; i<F.length()-1; i++) {
                    //... darf kein Zeichen auerhalb der Klammer stehen z.B. sin(12x+5)3
                    if (!innerhalbKlammer[i]) throw new Lesefehler(sp.getString("Parser_RechenzeichenKlammer"), F); 
                }
                //Wenn eine Klammer nicht DIREKT hinter dem Operator steht
                else if (F.indexOf('(', naechster[2])!=-1) throw new Lesefehler(sp.getString("Parser_Klammer1")+monOperatoren[naechster[0]]+sp.getString("Parser_Klammer2"), F);  //$NON-NLS-2$
                
                baum=new Monadisch(naechster[0]);
                //Wenn eine Multiplikation wie 2xsiny voransteht, dann wird dieser Baum nicht negativ, da die Punktrechnung dann negativ ist
                if (negativ && naechster[1]==0) baum.negativ();
                baum.haengeAn(lese(F.substring(naechster[2]), 4, false));
                letzterOp=naechster[1];
            
            
                //Wenn ein Faktor wie 2x siny voransteht
                if (naechster[1]!=0) {
                    Baum baumPkt;
                    baumPkt=new Punktrechnung();
                    if (negativ) baumPkt.negativ();
                    baumPkt.haengeAn(baum);
                    baumPkt.haengeAn(lese(F.substring(0, naechster[1]), 4, false));
                    return baumPkt;
                }
            
                //Rckgabe des Baumes, je nachdem, ob Multiplikation oder monadisch
                if (baum!=null) {return baum;}
            }
            
            
            
            
        //Exponenten
        case 4:
            if (F.indexOf('^')!=-1) {
                
                //Suche nach '^'
                int position=0;
                do {
                    position=F.indexOf('^', position);
                    if (position==-1) break;
                } while (innerhalbKlammer[position]);
                if (position==F.length()-1) throw new Lesefehler(sp.getString("Parser_Exponent"), F); 
                if (position==0) throw new Lesefehler(sp.getString("Parser_Basis"), F); 
                
                //Wenn kein ^ vorhanden, weiter bei case 5
                if (position!=-1) {
                    //Syntaxkontrolle, wenn Klammern hinter ^
                    if (F.charAt(position+1)=='(') for (int i=position+1; i<F.length()-1; i++) {
                        if (!innerhalbKlammer[i]) throw new Lesefehler(sp.getString("Parser_RechNachKlammer"), F); 
                    }
                    
                    //Wenn vor der Basis noch Faktoren stehen:
                    int beginnBasis=-1;
                    Punktrechnung baumPkt=null;
                    if (F.charAt(position-1)==')') {
                        //Suche nach dem Beginn der Basis. i=position-2, weil in position-1 ')' steht. Das fhrte zum Abbruch der Schleife 
                        for (int i=position-2; beginnBasis==-1; i--) {
                            //		v verhindert ArrayIndexOutOfBounds
                            if (i==-1 || !innerhalbKlammer[i]) beginnBasis=i+1;
                        }
                        if (beginnBasis!=0) {
                            baumPkt=new Punktrechnung();
                            if (negativ) baumPkt.negativ();
                            baumPkt.haengeAn(lese(F.substring(0, beginnBasis), 5, false));
                        }
                    }
                    else {
                        //Auf Deutsch: Wenn eine Zahl voransteht, weise beginnBasis i zu, sobald etwas anderes als eine Zahl gefunden wird.
                        if (F.charAt(position-1)<'0' && F.charAt(position-1)>'9') {for (int i=position-1; beginnBasis==-1; i--) if ((F.charAt(i)<'0' && F.charAt(i)>'9') && F.charAt(i)!=',') beginnBasis=i-1;}
                        //Ansonsten knnen nur noch Buchstaben voranstehen (Ausnahme "pi")
                        else {
                            if(F.charAt(position-1)=='i' && F.charAt(position-2)=='p') beginnBasis=position-2;
                        	else beginnBasis=position-1;
                        }
                        
                        if (beginnBasis!=0) {
                            baumPkt=new Punktrechnung();
                            if (negativ) baumPkt.negativ();
                            baumPkt.haengeAn(lese(F.substring(0, beginnBasis), 5, false));
                        }
                    }
                    
                    //Nicht Baum, sondern Exponential, weil Baum.haengeAn(a, b) nicht existiert. Dies ist kein aber Problem bei return, da Exponent Erbe von Baum.
                    Exponential exp=new Exponential();
                    if (negativ && baumPkt==null) exp.negativ();
                    //Anhngen						v basis												v exponent
                    if (baumPkt==null) exp.haengeAn(lese(F.substring(0, 		  position), 5, false), lese(F.substring(position+1), 5, false));
                    else 			   exp.haengeAn(lese(F.substring(beginnBasis, position), 5, false), lese(F.substring(position+1), 5, false));
                    if (baumPkt!=null) {
                        baumPkt.haengeAn(exp);
                        return baumPkt;
                    }
                    else return exp;
                }
            }
        
  
        
            
            
        //Verabeitung der Klammern
        case 5:
            //Weiter zu case 6, wenn keine Klammern vorhanden
            if (F.indexOf('(')!=-1) {
                boolean faktor=false;
                letzterOp=0;
                baum=new Punktrechnung();
                if (negativ) baum.negativ();
                
                if (!innerhalbKlammer[0]) faktor=true;
                for (int i=0; i<F.length(); i++) {
                    //Ein Faktor steht auerhalb der Klammer
                    if (faktor) {
                        //Sobald v == true, beginnt die Klammer
                        if (innerhalbKlammer[i]) {
                            //	v verhindert Anhngen von "" bei (x)(y)							 v baum ist schon negativ
                            if (letzterOp != i) baum.haengeAn(lese(F.substring(letzterOp, i), 6, false));
                            letzterOp=i;
                            faktor=false;
                        }
                    }
                    //Dies ist fr innerhalb der Klammer
                    else {
                        //Sobald v == true, ist die Klammer zu Ende
                        if (!innerhalbKlammer[i]) {
                            //Der Klammerausdruck wird angehngt	v '(' soll nicht bergeben werden
                            baum.haengeAn(lese(F.substring(letzterOp+1, i), 1, false));
                            //+1, denn das beim nchsten Teilen soll die Klammer nicht mitbergeben werden
                            letzterOp=i+1;
                            faktor=true;
                        }
                    }
                }
                return baum;
            }
            
            
        //Kleinste Elemente des Funktionsterms: 0,2x ...
        case 6:
            NumberFormat nf = NumberFormat.getInstance();
            ParsePosition pp =new ParsePosition(0);
            double d=1.0;
            char c='?';
            Baum baumPkt=null;
            //Fr den Fall, dass baum von den oberen cases noch eine Referenz enthlt
            baum=null;
            boolean gradMass=false;
            boolean amEndeDesTerms=false;
            
            
            for (int i=0; i<F.length(); i++) {
                //Parsen eines Faktors
                try { try {
                    	pp.setIndex(i);
                    	//NullPointerException, wenn keine Zahl vorhanden
                    	d=((Number)nf.parseObject(F, pp)).doubleValue();
                    	i=pp.getIndex();
                    	
                    	//StringIndexOutOfBoundsException, wenn eine Zahl am Ende des Terms stand. berspringt einige nun unntige Zeilen
                    	if (F.charAt(i)=='') {
                    	    d=Math.toRadians(d);
                    	    i++;
                    	}
                    	if (F.indexOf("pi")==i) { 
                    	    d=d*Math.PI;
                    	    i=i+2;
                    	}
                    	if (F.charAt(i)=='e') {
                    	    d=d*Math.E;
                    	    i++;
                    	}
                	}
                	//Wenn kein Faktor vor dem Parameter steht
                	catch (NullPointerException e) {
                	    i=pp.getErrorIndex();
                	    d=1.0;
                	    
                	    if (F.indexOf("pi")==i) { 
                	        d=d*Math.PI;
                	        i=i+2;
                	    }
                	    if (F.charAt(i)=='e') {
                	        d=d*Math.E;
                	        i++;
                	    }
                	}
                
                
                	//Parsen eines Parameters/der Variablen x
                	if (i<F.length() && (F.charAt(i)>'A'-1 && F.charAt(i)<'Z'+1) || F.charAt(i)=='x') {
                	    c=F.charAt(i);
                	    if (F.charAt(i+1)=='') {gradMass=true; i++;}
                	}
                	//Die Suche nach - ist nur ntig, wenn keine Zahl hinter dem - steht. Daher hier im ueren Try-Block
                	else if(F.charAt(i)=='-') { if (negativ) negativ=false; 
                								else negativ=true;
                	}
                	else if(F.charAt(i)!='e' || F.charAt(i)!='p' || F.charAt(i)<'0' || F.charAt(i)>'9') throw new Lesefehler(sp.getString("Parser_UnzZeichen")+F.charAt(i)+sp.getString("Parser_UnzZeichen2"), F);  //$NON-NLS-2$
                }
                //Kann auftreten, wenn i innerhalb der Schleife erhht wurde (weil z.B. PI am Ende des Terms stand)
                catch (StringIndexOutOfBoundsException e) {
                    amEndeDesTerms=true;
                }
            
                //Das Anhngen der Bume. Existieren mehrere Bltter (2x4y), mssen sie durch Punktrechnung verbunden werden
                if (baum==null && (amEndeDesTerms || F.charAt(i)!='-')) {baum=new Blatt(d, c); if (gradMass) ((Blatt)baum).gradMass(); if (negativ) baum.negativ();}
                //	^ Beim zweiten Durchlauf ist baum!=null
                else if (amEndeDesTerms || F.charAt(i)!='-'){
                    //Wenn noch keine Punktrechnung vorhanden ist:
                    if (baumPkt==null) {
                        baumPkt=new Punktrechnung();
                        if (negativ) baumPkt.negativ();
                        baumPkt.haengeAn(baum);
                    }
                    //Anhngen beliebig vieler Bltter an die Punktrechnung
                    baum=new Blatt(d, c);
                    if (negativ) baum.negativ();
                    baumPkt.haengeAn(baum);
                }
            }
            if (baumPkt!=null) {return baumPkt;}
            return baum;
            
            
        default: throw new Exception("Nicht untersttzter Typ im lese()-switch: "+typ); 
        }
    }
    
    
    /**Sucht den nchsten monadischen Operator.
     * @param F Der Term, der durchsucht wird.
     * @param innerhalbKlammer[]
     * @param offset
     * @return int[0] = Art des Operators,  int[1] = Position,  int[2] = Offset: Das nchste Zeichen gleich nach dem Operator
     */
    private int[] naechsterMonadischer(String F, boolean innerhalbKlammer[], int offset) {
        int position;
        int laenge;
        
        
        //Diese Schleife sucht den nchsten monadischen Operator jeder Art. Sie luft rckwrts, damit nicht nur sin aus dem String "arcsin" gefunden wird, sondern zuerst arcsin gesucht wird.
        for (int i=monOperatoren.length-1; i>-1; i--) {
            position=F.indexOf(monOperatoren[i], offset);
            if (position!=-1 && !innerhalbKlammer[position]) {
                switch (i) {
                case 0: case 1: case 2: laenge=3; break; //sin, cos, tan
                case 3: case 4: case 5: laenge=6; break; //arcsin ...
                case 6: laenge=2; break;				 //ln
                case 7: laenge=4; break;				 //sqrt
                default: laenge=-1; //Tritt nicht auf, da die Zhlschleife keine anderen Werte zulsst. Nur, damit der Compiler mitspielt.
                }
                return new int[] {
                        i, //Art des Operators
                        position,
                        position+laenge
            	};
            }
        }
        
        //Wenn es keinen monadischen Operator mehr gibt:
        return new int[] {
                -1,
                -1,
                -1
        };
    }
}

