package parser.baeume;


/**Enthlt zwei Bume: Basis und Exponent.
 * @author Stephan Hagebck
 */
class Exponential extends Baum {
    /**Diese Klasse kann nur zwei Bume referenzieren:*/
    protected Baum basis, exponent;
    
    
    /**Instanziert einen Baum, der Basis^Exponent zur Verfgung stellt.
     */
    public Exponential() {
        
    }
    
    
    /**Bei einem Exponenten muss man zwei Bume anhngen.
     * @param b Basis
     * @param e Exponent
     */
    void haengeAn(Baum b, Baum e) {
        basis=b;
        exponent=e;
    }
    
    
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#xVorhanden()
     */
    protected boolean xVorhanden() {
        if (basis.xVorhanden() || exponent.xVorhanden()) return true;
        else return false;
    }
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#haengeAn(parser.baeume.Baum)
     */
    void haengeAn(Baum baum) {
        //Nicht untersttzt
    }
    
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#f(double)
     */
    public double f(double x) {
        if (negativ)return Math.pow(-1*basis.f(x), exponent.f(x));
        return Math.pow(basis.f(x), exponent.f(x));
    }


    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        String s ='('+basis.toString()+")^("+exponent.toString()+')';
        if (negativ) s="-"+s;
        return s;
    }


    /* (non-Javadoc)
     * @see parser.baeume.Baum#leiteAb()
     */
    public Baum leiteAb() {
        Baum basisAbgel, expAbgel=null, innereAbl=null;
        Punktrechnung rueckPkt=null;
        
        //basis bleibt
        basisAbgel=(Baum)basis.clone();
        //Fr verkettete Funktionen
        if (!(basis instanceof Blatt)) innereAbl=basis.leiteAb();
        
        //Exponent -1
        if (!exponent.xVorhanden()) {
        	//Kein x vorhanden, also einfach 1 subtrahieren
            if (exponent instanceof Strichrechnung) expAbgel=(Baum)exponent.clone();
        	else {
        	    expAbgel=new Strichrechnung();
        	    expAbgel.haengeAn((Baum)exponent.clone());
        	}
        	expAbgel.haengeAn(new Blatt(-1.0, '?'));
        }
        
        else {
            /*g(x)^h(x) (wobei g und h komplexere Funktionen mit x sind): 
             *y' =	g^h	* (						) rueckPkt
             *				 		 +			  summe
             *			   h' *ln(g)	h * g'/g  
             *			   p1			p2
             */
            if (basis.xVorhanden()) {
                Baum g=(Baum)basis.clone();
                Baum h=(Baum)exponent.clone();
                
                Punktrechnung p1=new Punktrechnung();
                p1.haengeAn(h.leiteAb());
                Baum mon=new Monadisch(6); //Typ 6 ist ln
                mon.haengeAn(g);
                p1.haengeAn(mon);
                
                Punktrechnung p2=new Punktrechnung();
                p2.haengeAn(h);
                p2.haengeAn(g.leiteAb());
                p2.haengeAnDiv(g);
                
                Baum summe = new Strichrechnung();
                summe.haengeAn(p1);
                summe.haengeAn(p2);
                
                rueckPkt = new Punktrechnung();
                rueckPkt.haengeAn(this);
                rueckPkt.haengeAn(summe);
                
                return rueckPkt;
            }
            
            //Andernfalls ist es eine Exponentialfunktion. Ableitung ist die Funktion selber * ln(basis)
            else {
                rueckPkt=new Punktrechnung();
                if (negativ) rueckPkt.negativ();
                rueckPkt.haengeAn((Baum)this.clone());
                Monadisch mon=new Monadisch(6); //ln
                mon.haengeAn((Baum)basis.clone());
                rueckPkt.haengeAn(mon);
                if (innereAbl!=null) rueckPkt.haengeAn(innereAbl);
                return rueckPkt;
            }
        }
        
        
        
        //Rckgabe fr alle nicht-Exponentialfunktionen --> Exponent ohne x
        Exponential rueckgabe=new Exponential();
        rueckgabe.haengeAn(basisAbgel, expAbgel);
        
        rueckPkt=new Punktrechnung();
        if (negativ) rueckPkt.negativ();
        //Da der Exponent beim Ableiten vor die Basis geschrieben wird
        rueckPkt.haengeAn((Baum)exponent.clone());
        rueckPkt.haengeAn(rueckgabe);
        if (innereAbl!=null) rueckPkt.haengeAn(innereAbl);
        return rueckPkt;
    }
    
    
    /* (non-Javadoc)
	 * @see java.lang.Object#clone()
	 */
	public Object clone() {
	    Exponential rueckgabe=(Exponential)super.clone();
	    
	    rueckgabe.basis= (Baum)basis.clone();
	    rueckgabe.exponent=(Baum)exponent.clone();
	    //Primitive Datentypen werden ja kopiert...
	    return rueckgabe;
	}
	
	
	/* (non-Javadoc)
	 * @see parser.baeume.Baum#vereinfache()
	 */
	public Baum vereinfache() {
	    basis=basis.vereinfache();
	    exponent=exponent.vereinfache();
	    
	    //(...)^1 = (...)
	    if (exponent instanceof Blatt && ((Blatt)exponent).faktor==1.0 && ((Blatt)exponent).keinParameter) {
	        if (negativ) basis.negativ();
	        return basis;
	    }
	    //(...)^0 = 1
	    if (exponent instanceof Blatt && ((Blatt)exponent).faktor==0.0 && ((Blatt)exponent).keinParameter) {
	        if (negativ) return new Blatt(-1.0, '?');
	        else return new Blatt(1.0, '?');
	    }
	    
	    return this;
	}
}
