package parser.baeume;


/**Baum, der fr alle monadischen Operationen sorgt.
 * @author Stephan Hagebck
 */
class Monadisch extends Baum {
    /**Sorgt dafr, dass je nach Fall sin, cos... berechnet wird.*/
    private int typ;
    /**Da diese Klasse einen monadischen Operator reprsentiert, braucht sie nur einen Baum.*/
    private Baum baum;
    
    
    
    /**Instanziert einen Baum mit dem Funktionsumfang fr die monadischen Operatoren.
     *@param typ sin, cos...
     */
    public Monadisch(int typ) {
        this.typ=typ;
    }
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#haengeAn(parser.baeume.Baum)
     */
    public void haengeAn(Baum baum) {
        if (this.baum==null) this.baum=baum;
        else System.out.println("Versuch, 2. Baum anzuhngen");
    }
    
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#f(double)
     */
    public double f(double x) {
        double y;
        switch (typ) {
        	case 0: y= Math.sin(baum.f(x)); break;
        	case 1: y= Math.cos(baum.f(x)); break;
        	case 2: y= Math.tan(baum.f(x)); break;
        	case 3: y= Math.asin(baum.f(x)); break;
        	case 4: y= Math.acos(baum.f(x)); break;
        	case 5: y= Math.atan(baum.f(x)); break;
        	case 6: y= Math.log(baum.f(x)); break;
        	case 7: y= Math.sqrt(baum.f(x)); break;
        	default: System.out.println("Nicht untersttzter Typ in Monadisch.f(x)"); y= 0.0; break;
        }
        if (negativ) return y*-1;
        else return y;
    }


    /* (non-Javadoc)
     * @see parser.baeume.Baum#xVorhanden()
     */
    protected boolean xVorhanden() {
        return baum.xVorhanden();
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        String s=BaumErzeuger.monOperatoren[typ]+baum.toString();
        if (negativ) s="-"+s;
        return s;
    }

    /* (non-Javadoc)
     * @see parser.baeume.Baum#leiteAb()
     */
    public Baum leiteAb() {
        Baum innereAbl=null;
        Monadisch mon;
        Exponential exp, exp2;
        Strichrechnung str;
        Baum rueckgabe;
        
        //Fr Verkettete Funktionen
        if (!(baum instanceof Blatt)) innereAbl=baum.leiteAb();
        
        
        //Ableitung der verschiedenen Typen
        switch (typ) {
        	case 0: mon=new Monadisch(1); mon.haengeAn(baum); if (negativ) mon.negativ(); rueckgabe= mon; break;//sin
        	case 1: mon=new Monadisch(0); mon.haengeAn(baum); if (!negativ) mon.negativ(); rueckgabe= mon; break;//cos
        	//Laut meiner Formelsammlung ist tan'(x) ==  tan(x)+1 (sieht aus wie Pythagoras, denn es ist anscheinend auch 1/cos(x) )
        	case 2: mon=new Monadisch(2); mon.haengeAn(baum);
        			exp=new Exponential(); exp.haengeAn(mon, new Blatt(2.0, '?'));
        			str = new Strichrechnung(); str.haengeAn(exp); str.haengeAn(new Blatt(1.0, '?'));
        			if (negativ) str.negativ();
        			rueckgabe= str; break;//tan
        	// Die Ableitungsregel auf Deutsch (arcosx)' = 1/( sqrt(1-x) )
        	case 3: exp=new Exponential(); exp.haengeAn(baum, new Blatt(2.0, '?')); exp.negativ();
        	    	str=new Strichrechnung(); str.haengeAn(new Blatt(1.0, '?')); str.haengeAn(exp);
        	    	exp2=new Exponential(); exp2.haengeAn(str, new Blatt(-0.5, '?')); rueckgabe= exp2; break;//asin
    		//Die Ableitungsregel auf Deutsch (arcosx)' =- 1/( sqrt(1-x) )
        	case 4: exp=new Exponential(); exp.haengeAn(baum, new Blatt(2.0, '?')); exp.negativ();
	    			str=new Strichrechnung(); str.haengeAn(new Blatt(1.0, '?')); str.haengeAn(exp);
    				exp2=new Exponential(); exp2.haengeAn(str, new Blatt(-0.5, '?')); /* einziger Unterschied:*/ exp2.negativ(); rueckgabe= exp2; break;//acos
    		//Die Ableitungsregel auf Deutsch (arcosx)' = 1/( 1+x )
        	case 5: exp=new Exponential(); exp.haengeAn(baum, new Blatt(2.0, '?'));
	    	str=new Strichrechnung(); str.haengeAn(new Blatt(1.0, '?')); str.haengeAn(exp);
	    	exp2=new Exponential(); exp2.haengeAn(str, new Blatt(-1.0, '?')); rueckgabe= exp2; break;//atan
	    	//(lnx)' = 1/x
        	case 6: exp=new Exponential(); exp.haengeAn(baum, new Blatt(-1.0, '?')); rueckgabe= exp; break;//ln
        	//Ableitung der Wurzel durch Ableitung von x^(-0.5)
        	case 7:  exp=new Exponential(); exp.haengeAn(baum, new Blatt(0.5, '?')); rueckgabe= exp.leiteAb(); break;//sqrt
        	default: rueckgabe=null; break;//Tritt nicht auf.
        }
        
        if (innereAbl==null) return rueckgabe;
        Baum pkt=new Punktrechnung();
        pkt.haengeAn(rueckgabe);
        pkt.haengeAn(innereAbl);
        return pkt;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#clone()
     */
    public Object clone() {
        //denke, die Prozedur ist klar. Ein Datentyp ist zu klonen.
        Monadisch rueckgabe=(Monadisch)super.clone();
	    
	    rueckgabe.baum= (Baum)baum.clone();
	    return rueckgabe;
    }

    /* (non-Javadoc)
     * @see parser.baeume.Baum#vereinfache()
     */
    public Baum vereinfache() {
        
        return this;
    }
}
