package parser.baeume;

import java.util.ArrayList;

/**Prft eine Baumstruktur, ob sie ein Polynom reprsentiert.
 * @author Stephan Hagebck
 */
public class Polynompruefer {
    
    private double[] parameter;
    private boolean keinPolynom;
    
    /**Ein Konstruktor halt...
     */
    public Polynompruefer() {
        parameter=new double[11];
    }
    
       
    /**Prft das angegebene Polynom.
     * @param polynom
     * @return <b>double[11]</b> Enthlt die Faktoren, die vor x^5 [5] bzw. x^0 [0] stehen.
     */
    public double[] pruefePolynom(Baum polynom) {
        if (polynom instanceof Blatt) {
            Blatt blatt=(Blatt)polynom;
            if (blatt.parameter == 'x') {
                parameter[1]=blatt.faktor;
                parameter[0]=0.0;
            }
            else if (blatt.keinParameter) {
                parameter[0]=blatt.faktor;
            }
        }
        else if (polynom instanceof Punktrechnung) {
            pruefePunktrechnung((Punktrechnung)polynom);
        }
        else if (polynom instanceof Exponential) {
            try {
                parameter[pruefeExponent((Exponential)polynom)]=blattwert(((Exponential)polynom).basis);
            }
            catch (ArrayIndexOutOfBoundsException e) {
                keinPolynom=true;
            }
        }
        else if ((polynom instanceof Strichrechnung)) {
            ArrayList liste=polynom.liste;
            Object element;
            for (int i=0; i<liste.size(); i++) {
                element = liste.get(i);
                if (element instanceof Punktrechnung) pruefePunktrechnung((Punktrechnung)element);
                else if (element instanceof Exponential) pruefeExponent((Exponential)element);
                else if (element instanceof Blatt) if (((Blatt)element).xVorhanden()) parameter[1]=parameter[1]+ blattwert(element);
                							  else parameter[0]= parameter[0]+blattwert(element);
                else if (element instanceof Monadisch) keinPolynom=true;
            }
        }
        else return null;
        //Wenn keiner der Flle zutrifft
        if (keinPolynom) return null;
        else return parameter;
    }
    
    
    /**Wie der Name schon sagt...
     * @param exp
     * @return Index des Exponenten
     */
    private int pruefeExponent(Exponential exp) {
        int d=0;
        try {
            if (((Blatt)exp.basis).parameter!='x') keinPolynom=true;
            if (!((Blatt)exp.exponent).keinParameter) keinPolynom=true;
            d=(int)blattwert(exp.exponent);
            //Kein ganzzahliger Exponent
            if (d != blattwert(exp.exponent)) keinPolynom=true;
        }
        //Falls keine Bltter (=x oder 1,2,3) auf basis oder exponent stehen.
        catch (ClassCastException e) {
            keinPolynom=true;
        }
        return d;
    }

    /**Wahrscheinlich auch selbsterklrend...
     * @param p
     */
    private void pruefePunktrechnung(Punktrechnung p) {
        ArrayList liste=p.liste;
        double faktor=1;
        int exponent=0;
        boolean expGefunden=false;
        for (int i=0; i<liste.size(); i++) {
            if (liste.get(i) instanceof Exponential) {
                if (expGefunden) keinPolynom=true;
                expGefunden=true;
                exponent=pruefeExponent((Exponential) liste.get(i));
            }
            else if (liste.get(i) instanceof Blatt ) {
                Blatt b= (Blatt)liste.get(i);
                if (b.keinParameter) faktor=faktor*blattwert(b);
                else if (b.xVorhanden) {
                    expGefunden=true;
                    exponent=1;
                    faktor=faktor*blattwert(b);
                }
                else keinPolynom=true;
            }
            else keinPolynom=true;
        }
        try {
            parameter[exponent]=faktor;
        }
        catch (ArrayIndexOutOfBoundsException e) {
            keinPolynom=true;
        }
    }
    
    
    
    /**Liest den Faktor eines Blattes aus.
     * @param blatt
     * @return Den Faktor
     */
    private double blattwert(Object blatt) {
        Blatt b=(Blatt)blatt;
        return b.faktor;
    }
}
