package parser.baeume;

import java.util.ArrayList;


/**Stellt alle Funktionen fr die Punktrechnung zur Verfgung.
 * @author Stephan Hagebck
 */
class Punktrechnung extends Baum {
    /**Enthlt die Bume, die dividiert werden mssen*/
    protected ArrayList listeDiv;
   
    /**Instanziert einen Baum mit den Eigenschaften der Punktrechnung.
     */
    Punktrechnung() {
        liste=new ArrayList();
        listeDiv=new ArrayList();
    }
    
    
    /**Wenn ein Baum nicht multipliziert, sondern dividiert werden soll
     * @param baum zum Anhngen
     */
    void haengeAnDiv(Baum baum) {
        listeDiv.add(baum);
    }
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#f(double)
     */
    public double f(double x) {
        double d=1.0;
        for (int i=0; i<liste.size(); i++) {
            d=d*((Baum)liste.get(i)).f(x);
        }
        for (int i=0; i<listeDiv.size(); i++) {
            d=d/((Baum)listeDiv.get(i)).f(x);
        }
        if (negativ) d=d*-1;
        return d;
    }
    
    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#xVorhanden()
     */
    protected boolean xVorhanden() {
        if (super.xVorhanden()) return true;
        
        Baum b;
	    boolean xGefunden=false;
	    
	    for (int i=0; i<listeDiv.size() && !xGefunden; i++) {
	        b=(Baum)listeDiv.get(i);
	        if (b.xVorhanden()) xGefunden=true;
	    }
	    return xGefunden;
    }
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#toString()
     */
    public String toString() {
        String s="";
        for (int i=liste.size()-1; i>-1; i--) {
            s=liste.get(i).toString()+'*'+s;
        }
        if (liste.size()!=0) s='('+s.substring(0, s.length()-1)+')';
        
        if (listeDiv.size()!=0) s=s+'/'+'(';
        for (int i=listeDiv.size()-1; i>-1; i--) {
            s=s+listeDiv.get(i).toString()+'*';
        }
        s=s.substring(0, s.length()-1)+')';
        if (negativ) s="-"+s;
        return s;
    }

    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#leiteAb()
     */
    public Baum leiteAb() {
        Baum baum;
        ArrayList faktoren=new ArrayList();
        ArrayList xBaeume=new ArrayList();
        ArrayList faktorenDiv=new ArrayList();
        ArrayList xBaeumeDiv=new ArrayList();
        
        for (int i=0; i<liste.size(); i++) {
            baum= (Baum)liste.get(i);
            if (!baum.xVorhanden()) faktoren.add(baum);
            else xBaeume.add(baum);
        }
        
        for (int i=0; i<listeDiv.size(); i++) {
            baum= (Baum)listeDiv.get(i);
            if (!baum.xVorhanden()) faktorenDiv.add(baum);
            else xBaeumeDiv.add(baum);
        }
        
        
        //Entscheidung fllt zwischen Quotientenregel oder Produktregel
        if (xBaeumeDiv.size()==0) { //Produktregel
            Punktrechnung rueckgabe=new Punktrechnung();
            //Leere Listen lassen liste unverndert.
            rueckgabe.liste.addAll(faktoren);
            rueckgabe.listeDiv.addAll(faktorenDiv);
            
            Strichrechnung str=new Strichrechnung();
            Punktrechnung pkt;
            for (int i=0; i<xBaeume.size(); i++) {
                pkt=new Punktrechnung();
                pkt.haengeAn(((Baum)xBaeume.get(i)).leiteAb());
                for (int j=0; j<xBaeume.size(); j++) {
                    if (j!=i) pkt.haengeAn((Baum)(xBaeume.get(j)));
                }
                str.haengeAn(pkt);
            }
            
            if (faktoren.size()==0 && faktorenDiv.size()==0) return str;
            rueckgabe.haengeAn(str);
            return rueckgabe;
        }
        //Quotientenregel f(x) / g(x)
        //else 
            Punktrechnung f=new Punktrechnung();
            f.liste.addAll(this.liste);
            
            Punktrechnung g=new Punktrechnung();
            g.liste.addAll(this.listeDiv);
            
            
            Punktrechnung rueckgabe=new Punktrechnung();
            
            	Exponential exp=new Exponential();
            	exp.haengeAn(g, new Blatt(2.0, '?'));
        	rueckgabe.haengeAnDiv(exp); //Nenner
            
            	Strichrechnung str=new Strichrechnung();
            
            		Punktrechnung p1=new Punktrechnung();
        			p1.liste.addAll(g.liste);
        			p1.haengeAn(f.leiteAb());
    			str.haengeAn(p1); //g(x) * f'(x)
            
        			Punktrechnung p2=new Punktrechnung();
        			p2.liste.addAll(f.liste);
        			p2.haengeAn(g.leiteAb());
        			p2.negativ();
    			str.haengeAn(p2); //-(g'(x) * f(x))
    			
			rueckgabe.haengeAn(str); //p1-p2 / g
			return rueckgabe;
    }
    
    
    
    /* (non-Javadoc)
     * @see java.lang.Object#clone()
     */
    public Object clone() {
        Punktrechnung rueckgabe=(Punktrechnung)super.clone();
        rueckgabe.listeDiv=(ArrayList)listeDiv.clone();
        return rueckgabe;
    }


    
    /* (non-Javadoc)
     * @see parser.baeume.Baum#vereinfache()
     */
    public Baum vereinfache() {
        //Sollte diese Punktrechnung nur ein Element enthalten, fllt sie weg.
        if (liste.size()==1 && listeDiv.size()==0) {
            Baum baum=(Baum)liste.get(0);
            if (negativ) baum.negativ();
            return baum;
        }
        
        Baum baum;
        boolean weiter;
        
        do {
        weiter=false;
        double y=1.0;
        
        for (int i=0; i<liste.size(); i++) {
            baum=(Baum)liste.get(i);
            //Alle Faktoren werden miteinander multipliziert und zu einem zusammengefasst
            if (baum instanceof Blatt && ((Blatt)baum).keinParameter) {
                Blatt b=(Blatt)baum;
                y=y*b.faktor;
                liste.remove(i--);
            }
            else if (baum instanceof Punktrechnung) {
                //Zwei Punktrechnungen zusammenfhren
                Punktrechnung baumPkt =(Punktrechnung) baum;
                liste.remove(i--);
                liste.addAll(baumPkt.liste);
                listeDiv.addAll(baumPkt.listeDiv);
                
                //schaltet negativ an oder aus
                if (baumPkt.negativ) this.negativ();
            }
            //andere Elemente kann man vielleicht vereinfachen
            else liste.set(i, baum.vereinfache());
            //TODO vielleicht spter einmal exponenten zusammenfassen
        }
        
        //Die zusammengefassten Faktoren mssen wieder angehngt werden.
        if (y!=1.0) liste.add(0, new Blatt(y, '?'));
        
        
        //Selbes Verfahren beim Vereinfachen des Nenners
        y=1.0;
        
        for (int i=0; i<listeDiv.size(); i++) {
            baum=(Baum)listeDiv.get(i);
            //Alle Faktoren werden miteinander multipliziert und zu einem zusammengefasst
            if (baum instanceof Blatt && ((Blatt)baum).keinParameter) {
                Blatt b=(Blatt)baum;
                y=y*b.faktor;
                listeDiv.remove(i--);
            }
            else if (baum instanceof Punktrechnung) {
                //Zwei Punktrechnungen zusammenfhren
                Punktrechnung baumPkt =(Punktrechnung) baum;
                listeDiv.remove(i--);
                
                //Da hier eine Punktrechnung im Nenner angehngt wird, mssen liste und listeDiv getauscht werden.
                this.liste.addAll(baumPkt.listeDiv);
                this.listeDiv.addAll(baumPkt.liste);
                //Wenn liste erweitert wird, kann man vielleicht nochmal vereinfachen
                weiter=true;
                
                //schaltet negativ an oder aus
                if (baumPkt.negativ) this.negativ();
            }
            //andere Elemente kann man vielleicht vereinfachen
            else listeDiv.set(i, baum.vereinfache());
            //TODO vielleicht spter einmal exponenten zusammenfassen
        }
        
        //Die zusammengefassten Faktoren mssen wieder angehngt werden.
        if (y!=1.0) listeDiv.add(0, new Blatt(y, '?'));
        } while (weiter);
        
        //Rckgabe des Vereinfachten Baumes
        return this;
    }
}
